<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// This function creates an array of debug messages
function debug_msg($debug_level, $debug_message)
{
	global $debug_msga;

	if (check_debuglevel($debug_level)) {
		// We need backtrace to get the calling function name
		$backtrace = debug_backtrace();
		if (isset($backtrace[1])) {
			$function_name = $backtrace[1]['function'];
		} else {
			$function_name = '';
		}
		$function_file = $backtrace[0]['file'];
		$function_line = $backtrace[0]['line'];

		$debugtext = '[' . $debug_level . '] ';
		if (isset($function_name)) {
			$debugtext .= $function_name . ': ';
		}
		$debugtext .= $debug_message;

		if (isset($function_file)) {
			$debugdscr = $function_file . ' on line ' .
				$function_line;
		}

		// Append debug message to array
		$debug_msga[] = $debugdscr;
		$debug_msga[] = $debugtext;
		$debug_msga[] = "##";

	}
}

// This function creates the actual debug messages in the dom object
function create_debug_msg($debug_msga)
{
	// Only show debug messages if we have at least one array element
	// which is not empty
	if (isset($debug_msga) && is_array($debug_msga) &&
	    isset($debug_msga[0]) && $debug_msga[0]) {
		domchild(2, 'div', array('id' => 'debug'), '');
		domchild(3, 'div', '', '######## DEBUG_OUTPUT_START ########');
		foreach ($debug_msga as $msg) {
			domchild(3, 'div', '', $msg);
		}
		domchild(3, 'div', '', '######### DEBUG_OUTPUT_END #########');
	}
}

// This function creates the actual debug messages when used in command line mode
function create_debug_console_msg($debug_msga)
{
	// Only show debug messages if we have at least one array element
	// which is not empty
	if (isset($debug_msga) && is_array($debug_msga) &&
	    isset($debug_msga[0]) && $debug_msga[0]) {
		print('######## DEBUG_OUTPUT_START ########' . "\n");
		foreach ($debug_msga as $msg) {
			print($msg . "\n");
		}
		print('######### DEBUG_OUTPUT_END #########' . "\n");
	}
}

// We have to check if the current global debug level and the level passed
// to this function fit together. Therefor we have to do a binary bitwise
// AND operation.
function check_debuglevel($debug_level)
{
	$DEBUG = getDebug();

	// This is our binary bitwise and operation check
	if (isset($DEBUG) && ($DEBUG & $debug_level) == true) {
		return(true);
	} else {
		return(false);
	}
}

?>
