<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

require_once('setget.inc.php');
require_once('sql.inc.php');
require_once('xhtml.inc.php');
require_once('pmacct.inc.php');
require_once('rrdtool.inc.php');
require_once('debug.inc.php');

function error_msg($error_message)
{
	global $error_msga;

	// We need backtrace to get the calling function name
	$backtrace = debug_backtrace();
	$function_file = $backtrace[0]['file'];
	$function_line = $backtrace[0]['line'];

	$errortext = $error_message;

	if (isset($function_file)) {
		$errordscr = $function_file . ' on line ' .
			$function_line;
	}

	// Append error message to array
	$error_msga[] = $errordscr;
	$error_msga[] = $errortext;
	$error_msga[] = "________________________________________";
}

function create_error_msg()
{
	global $error_msga;
	global $dom;

	if (isset($error_msga) && $error_msga) {
		// Search for tag div with attribute id=error to put error
		// messages above normal site content
		$params = $dom[0]->getElementsByTagName('div');
		foreach ($params as $i => $value) {
			if ($params->item($i)->getAttribute('id') == 'error' ) {
				foreach ($error_msga as $msg) {
					// Add errror message with p tag to
					// have correct css formating
					$domchild = $dom[0]->createElement('p');
					$params->item($i)->appendChild($domchild);

					// Add error message to p tag
					$domchild_text =
						$dom[0]->createTextNode($msg);
					$domchild->appendChild($domchild_text);
				}

			
				break;
			}	
		}

	}
}

function create_error_console_msg($error_msga)
{
	if (isset($error_msga) && $error_msga) {
		print('######## ERROR_OUTPUT_START ########' . "\n");
		foreach ($error_msga as $msg) {
			print($msg . "\n");
		}
		print('######## ERROR_OUTPUT_END ########' . "\n");
	}
}

// To integrate error messages in our xhtml file we need our
// own error handler
function error_handler($type, $message, $file, $line)
{

	global $error_msga;

	// Ignore E_STRICT (2048) errors, because PHP Pear is broken
	// /usr/share/php/DB.php on line 594
	if ($type != 2048) {
		//$error_msga[] = 'error type:' . $type;
		$error_msga[] = $file . ' on line ' . $line;
		$error_msga[] = strip_tags($message);
		$error_msga[] = '___________________________';

		// if headers are already send or the programm runs on the
		// command line we cannot use the error_msga and have to print
		// the error message directly
		if (runs_on_command_line()) {
			print($file . ' on line ' . $line . "\n");
			print(strip_tags($message) . "\n");
			print('___________________________' . "\n");
		} else if (headers_sent()) {
			print($file . ' on line ' . $line . "<br />\n");
			print(strip_tags($message) . "<br />\n");
			print('___________________________' . "<br />\n");
		}
	}

}

// To show not printed error messages we need our exception handler
function exception_handler($exception)
{

	global $error_msga;
	global $dom;

	$error_msga[] = 'Exception: '. $exception->getMessage();

	// if headers are already send or the programm runs on the command line
	// we cannot use the show_html_page function have to print the error
	// message directly
	if (runs_on_command_line()) {
		if (isset($error_msga)) {
			// Print all error messages
			foreach ($error_msga as $msg) {
				print($msg . "\n");
			}
		}
	} else if (headers_sent()) {
		if (isset($error_msga)) {
			// Print all error messages
			foreach ($error_msga as $msg) {
				print($msg . "<br />\n");
			}
		}
	} else {
		show_html_page($dom[0]);
	}

	// Destroy the php session
	session_destroy();

}

// To have more control of pear error message we have our own error handler
function pear_error_handler($err)
{

	global $error_msga;

	$error_msga[] = $err->getMessage();
	$error_msga[] = '___________________________';

	// we have to throw a exception at this point, because otherwise
	// the PEAR_Exception function suppress our error messages from the
	// error_msga array
	throw new Exception('pear_error');

}

function initialization()
{
	// We need our own error handler
	set_error_handler('error_handler');

	// We need our own exception handler
	set_exception_handler('exception_handler');

	// Set session name to id
	session_name('id');

	// Start php session
	session_start();

	// Set default value for some variables
	default_var_value();

	// Create html_dom
	html_dom();

	debug_msg(1, 'debuging is turned on with debuglevel: ' . getDebug());

	// Check if browser accepts xhtml
	if (isset($_SERVER['HTTP_ACCEPT'])) {
		if (preg_match('#application/xhtml\+xml#i',
				$_SERVER['HTTP_ACCEPT'])) {
			debug_msg(1, 'browser accepts xhtml');
		}
	}

	// Read configuration file
	readconfig();

	// Only use PEAR when we don't have a mysql and postgresql database
	if (getDatabaseType() != 'mysql' && getDatabaseType() != 'pgsql') {
		// for the PEAR_Error class
		require_once('PEAR.php');
		require_once('DB.php');

		// We need our own error handler for PEAR
		PEAR::setErrorHandling(PEAR_ERROR_CALLBACK,
			'pear_error_handler');
	}

}

// Set default value for some variables
function default_var_value()
{
	global $debug_msga;
	global $error_msga;

	$debug_msga = array();
	$error_msga = array();

}

// This functions syncs the navigation points between GET and SESSION context
function navigation_sync($submenu)
{

	if (isset($_GET['nav'])) {
		setNav($_GET['nav']);
		// Reset subnav after page change, but only if nav point
		// is a valid index for submenu
		if (isset($submenu[getNav()])) {
			setSubnav($submenu[getNav()][0]);
		}
	}

	if (isset($_GET['snav'])) {
		setSubnav($_GET['snav']);
	}

}

function guess_include_file($topmenu, $submenu)
{
	# Only alnum characters, hyphen and spaces are allowed for nav and snav
	$regex = '/^[a-z0-9\- ]+$/i';

	// Check if nav is set and matches $regex regular expression
	// if not include the default topmenu
	if (preg_match($regex, getNav())) {
		$include_file = 'include/' . strtolower(getNav()) . '/' .
			strtolower(getNav()) . '.inc.php';
	} else {
		error_msg('Navpoint "' . getNav() . '" is not valid');
		$include_file = 'include/' . strtolower($topmenu[0]) . '/' .
			strtolower($topmenu[0]) . '.inc.php';
		// Reset nav and subnav
		setNav($topmenu[0]);
		setSubnav($submenu[getNav()][0]);
	}

	// Check if snav is set and matches $regex regular expression
	if (preg_match($regex, getSubnav())) {
		// Check if snav is not the first submenu entry, because
		// we then include the main file
		if (getSubnav() != $submenu[getNav()][0]) {
			$include_file = 'include/' . strtolower(getNav()) . '/' .
				strtolower(getSubnav()) . '.inc.php';
		}
	} else {
		error_msg('Subnavpoint "' . getSubnav() . '" is not valid');
	}

	// replace whitespace to '_'
	setIncludeFile(preg_replace('/ /', '_', $include_file));
	debug_msg(1, 'include_file = ' . $include_file);

}
	
// add html_header to dom
function html_header($project_name)
{

	$sidetitle = $project_name . ' - ' . getNav() . ' page';

	// create head element
	domchild(1, 'head', '', '');

	// create title element
	domchild(2, 'title', '', $sidetitle);

	// include css stylesheet
	// <link rel="stylesheet" type="text/css" href="default.css">
	domchild(2, 'link', array('rel' => 'stylesheet', 'type' => 'text/css',
		'href' => 'default.css'), '');

	// include javascript files
	// global js file
	domchild(2, 'script', array('src' => 'js/global.js',
		'type' => 'text/javascript'), '');
	// sajax framework
	domchild(2, 'script', array('src' => 'js/sajax.php?id=' . session_id(),
		'type' => 'text/javascript'), '');
	// include file specific javascript code
	// replace .inc.php extension to .js extension
	$jsfile = preg_replace('/\.inc\.php$/', '.js', getIncludeFile());
	if (file_exists($jsfile)) {
		domchild(2, 'script', array('src' => $jsfile, 
			 'type' => 'text/javascript'), '');
	}

	// create body
	domchild(1, 'body', '', '');

	// header area
	domchild(2, 'div', array('id' => 'header'), '');

	// add project name
	domchild(3, 'h1', '', $project_name);

}

// Include html_menu files
function html_menu($topmenu, $submenu)
{
	include('include/menu/topmenu.inc.php');
	include('include/menu/leftmenu.inc.php');
}

function html_content($file)
{
	global $cfg;

	// content area
	domchild(2, 'div', array('id' => 'content'), '');

	// error messages area
	domchild(3, 'div', array('id' => 'error'), '');

	// error message when javascript is not enabled
	domchild(4, 'noscript', '', '');
	domchild(5, 'p', '', 'Please enable JavaScript');

	if (file_exists($file)) {
		include($file);
	} else {
		error_msg('No such file: ' . $file);
	}

}

function html_footer($footer)
{

	if (isset($footer) && $footer) {
		domchild(2, 'div', array('id' => 'footer'), $footer);
	}

}

function readconfig()
{
	global $cfg;
	global $domcfg;

	$cfgfile = getCfgfile();
	$cfg = array();

	$domcfg = new DOMDocument();
	$domcfg->encoding = 'UTF-8';
	$domcfg->version = '1.0';
	$domcfg->formatOutput = true;
	$domcfg->preserveWhiteSpace = false;
	if (file_exists($cfgfile)) {
		$domcfg->load($cfgfile);
	} else {
		// create new empty configuration
		$newconf = $domcfg->createElement('conf');
		$domcfg->appendChild($newconf);
	}
	$cfg = simplexml_import_dom($domcfg);

}

function writeconfig()
{
	global $domcfg;

	$cfgfile = getCfgfile();
	$retval = false;

	if (can_write($cfgfile)) {
		// writing the dom-object into the configuration file (as xml)
		if ($domcfg->save($cfgfile)) {
			$retval = true;
		}
	} else {
		error_msg($cfgfile . ' is not writable');
	}

	return($retval);
}

// Only strip slashes when get_magic_quotes_gpc is set
function _stripslashes($value)
{
	if (get_magic_quotes_gpc() == true) {
		return(stripslashes($value));
	} else {
		return($value);
	}
}

// Add attribute to a specific XML dom element
function add_attr2dom($tag, $attr, $val)
{
	global $dom;

	debug_msg(2, '$tag=' . $tag);
	debug_msg(2, '$attr=' . $attr);
	debug_msg(2, '$val=' . $val);

	$elements = $dom[0]->getElementsByTagName($tag);
	if (isset($elements) && $elements->length > 0) {
		debug_msg(2, '$elements->length=' . $elements->length);
		// Add attribute to every element found in dom
		foreach ($elements as $i => $value) {
			// check if we have a DOMElement
			if ($elements->item($i) instanceof DOMElement) {
				$dom_attr = $dom[0]->createAttribute($attr);
				$elements->item($i)->appendChild($dom_attr);

				$dom_val = $dom[0]->createTextNode($val);
				$dom_attr->appendChild($dom_val);
				debug_msg(2, 'true');
			}
		}
	}
}

// Add page refresh to site
function page_refresh($seconds)
{
	global $dom;

	$elements = $dom[0]->getElementsByTagName('head');
	if (isset($elements) && $elements->length > 0) {
		debug_msg(2, '$elements->length=' . $elements->length);
		// Add new child to every element found in dom
		foreach ($elements as $i => $value) {
			// check if we have a DOMElement
			if ($elements->item($i) instanceof DOMElement) {
				$dom_element = $dom[0]->createElement('meta');
				$elements->item($i)->appendChild($dom_element);

				$dom_attr = $dom[0]->createAttribute('http-equiv');
				$dom_element->appendChild($dom_attr);
				$dom_val = $dom[0]->createTextNode('refresh');
				$dom_attr->appendChild($dom_val);

				$dom_attr = $dom[0]->createAttribute('content');
				$dom_element->appendChild($dom_attr);
				$dom_val = $dom[0]->createTextNode($seconds);
				$dom_attr->appendChild($dom_val);

				debug_msg(2, 'true');
			}
		}
	}

}

// read interface counter from $file an return array with the counter values
// $retval['dev'] = Interface
// $retval['maxrxrate'] = Maximum interface speed (Resceive)
// $retval['maxtxrate'] = Maximum interface speed (Trasmit)
// $retval['rxbytes'] = Received bytes
// $retval['rxpackets'] = Received packets
// $retval['rxerrors'] = Received errors
// $retval['rxmulti'] = Received multicast
// $retval['txbytes'] = Transmited bytes
// $retval['txpackets'] = Transmited packets
// $retval['txerrors'] = Transmited errors
// $retval['interval'] = Refresh interval
// $retval['txrate'] = Transmit rate
// $retval['rxrate'] = Receive rate
// $retval['txpercent'] = Percentage transmit rate
// $retval['rxpercent'] = Percentage receive rate
// $retval['last'] = When was the function called (unixtimestemp)
function interface_counter()
{
	$dev = getInterface();
	$file = '/proc/net/dev';
	$retval = array('dev' => $dev,
			'maxrxrate' => getEtherRXSpeed($dev),
			'maxtxrate' => getEtherTXSpeed($dev));
	$regex = '/^\s*' . $dev . ':\s*/i';
	// default interval in seconds
	$interval = 2;

	if (file_exists($file)) {
		// read entire file into an array.
		$lines = file($file);
		// loop through the lines
		foreach ($lines as $line) {
			// search for $dev
			if (preg_match($regex, $line, $match)) {
				// cut off interface name
				$counters = preg_replace($regex, '', $line);
				// split counters
				$countera = preg_split('/\s+/', $counters);
				debug_msg(2, '$counters = ' . $counters);
				$retval['rxbytes'] = $countera[0];
				$retval['rxpackets'] = $countera[1];
				$retval['rxerrors'] = $countera[2];
				$retval['rxmulti'] = $countera[7];
				$retval['txbytes'] = $countera[8];
				$retval['txpackets'] = $countera[9];
				$retval['txerrors'] = $countera[10];
				$retval['interval'] = $interval;
			}
		}
	} else {
		error_msg($file . ': No such file or directory');
	}

	// check if we have icounters in our session
	if (isset($_SESSION['icounters']) && is_array($_SESSION['icounters'])) {
		// calculate rate interval
		$interval = microtime(1) - $_SESSION['icounters']['last'];
		// if the last call of this function is not more then one
		// second ago sleep to the next second
		if ($interval < 1) {
			sleep_to_next_second();
			// save new interval
			$interval = microtime(1) -
				$_SESSION['icounters']['last'];
		}
		// do the rate calculation
		$rxbyte_last = $_SESSION['icounters']['rxbytes'];
		$rxbyte_now = $retval['rxbytes'];
		// check if we have a counter overrun
		if ($rxbyte_now < $rxbyte_last) {
			debug_msg(1, 'rx counter overrun');
			// on 32 bit systems overrun is 2^32
			if ($rxbyte_last <= pow(2, 32)) {
				$rxstat = pow(2, 32) - $rxbyte_last +
					$rxbyte_now;
				debug_msg(1, 'rx counter overrun 32 bit');
			// on 64 bit systems overrun is 2^64
			} else if ($rxbyte_last <= pow(2,64)) {
				$rxstat = pow(2, 64) - $rxbyte_last +
					$rxbyte_now;
				debug_msg(1, 'rx counter overrun 64 bit');
			}
		} else {
			$rxstat = $rxbyte_now - $rxbyte_last;
		}
		$retval['rxrate'] = round($rxstat / $interval * 8);
		$txbyte_last = $_SESSION['icounters']['txbytes'];
		$txbyte_now = $retval['txbytes'];
		if ($txbyte_now < $txbyte_last) {
			debug_msg(1, 'tx counter overrun');
			// on 32 bit systems overrun is 2^32
			if ($txbyte_last <= pow(2, 32)) {
				$txstat = pow(2, 32) - $txbyte_last +
					$txbyte_now;
				debug_msg(1, 'tx counter overrun 32 bit');
			// on 64 bit systems overrun is 2^64
			} else if ($txbyte_last <= pow(2,64)) {
				$txstat = pow(2, 64) - $txbyte_last +
					$txbyte_now;
				debug_msg(1, 'tx counter overrun 64 bit');
			}
		} else {
			$txstat = $txbyte_now - $txbyte_last;
		}
		$retval['txrate'] = round($txstat / $interval * 8);

		// calculate percentage usage
		if ($retval['maxrxrate'] > 0) {
			$rxpercent = round($retval['rxrate'] /
					   $retval['maxrxrate'] * 100);
		} else {
			$rxpercent = 0;
		}
		if ($retval['maxtxrate'] > 0) {
			$txpercent = round($retval['txrate'] /
					   $retval['maxtxrate'] * 100);
		} else {
			$txpercent = 0;
		}

		if ($rxpercent > 100) {
			$rxpercent = 100;
		}
		if ($txpercent > 100) {
			$txpercent = 100;
		}
		$retval['rxpercent'] = $rxpercent;
		$retval['txpercent'] = $txpercent;
	} else {
		$retval['rxrate'] = 0;
		$retval['txrate'] = 0;
		$retval['rxpercent'] = 0;
		$retval['txpercent'] = 0;
	}

	// save current unix timestamp
	$retval['last'] = microtime(1);

	// save values in session to do rate calculations
	$_SESSION['icounters'] = $retval;

	return($retval);

}

// function to simplify svg integration
// include codebase for Internet Explorer ?
// 'codebase' =>
// 'http://download.adobe.com/pub/adobe/magic/svgviewer/win/3.x/3.03/en/SVGView.exe
function svg($level, $element, $width, $height)
{

	domchild($level, 'object', array('data' => $element,
					 'width' => $width, 'height' => $height,
					 'type' => 'image/svg+xml'),
		 'Please download the SVG viewer plugin from the Adobe' .
		 ' website. (http://www.adobe.com/svg/viewer/install/)');

	domchild($level+1, 'param', array('name' => 'src',
					  'value' => $element), '');

	// Does this code break the adobe viewer plugin ?
	domchild($level+1, 'a', array('href' => 
				'http://www.adobe.com/svg/viewer/install/'),
		 '(Link)');

}

// Check if file and corresponding directory is writeable
function can_write($file)
{
	$is_writeable = FALSE;
	$realfile = realpath($file);

	if (file_exists($file)) {
		debug_msg(1, $file . ' exists');
		if (is_writable($file)) {
			debug_msg(1, $file . ' is writeable');
			$is_writeable = TRUE;
		} else {
			error_msg($file . ' is not writeable');
		}
	} else {
		debug_msg(1, $file . ' does not exists');
		if (is_dir(dirname($file))) {
			if (is_writable(dirname($file))) {
				$is_writeable = TRUE;
			} else {
				error_msg('Cannot write to ' .
					dirname($file) .
					': Permission denied');
			}
		} else {
			error_msg(dirname($file) . ': No such directory');
		}
	}

	return $is_writeable;
}

// run a programm with $options and return output as array
function exec_cmd($command, $options)
{
	$output = array();

	debug_msg(2, '$command = ' . $command);
	debug_msg(2, '$options = ' . $options);

	exec($command . ' ' . $options, $output, $return_var);

	if ($return_var != 0) {
		error_msg('Return code of command "' . $command . ' ' .
			  $options . '" : ' . $return_var);
	}

	return($output);
}

// $columns = array of columns name (Name1, Name2, Name...)
// $rows = array of row values (0 => array(column_name1 => column_value1,
//                                         column_name2 => column_value2),
//                              1 => array(column_name1 => column_value1,
//                                         column_name2 => column_value2)...
function form1($level, $columns, $rows)
{
	$count = 0;

	domchild($level, 'form', array('action' => getSelf(),
		 'method' => 'post'), '');
	domchild($level+1, 'table', array('border' => '0'), '');
	domchild($level+2, 'tr', '', '');
	domchild($level+3, 'th', '', 'No.');

	foreach ($columns as $column) {
		domchild($level+3, 'th', '', $column);
	}
	// Extra table column for the delete button
	domchild($level+3, 'th', '', '');

	foreach ($rows as $row_key => $row) {

		domchild($level+2, 'tr', '', '');
		domchild($level+3, 'td', '', '');
		domchild($level+4, 'span', '', $count+1 . '.');

		foreach ($row as $column_key => $column) {

			domchild($level+3, 'td', '', '');
			domchild($level+4, 'input', array('type' => 'text',
				 'name' => $column_key . '[]',
				 'value' => _stripslashes($column)), '');

		}

		// if we have more headings then colums in $row we have to fill
		// them up
		if (sizeof($columns) > sizeof($row)) {
			for ($i=sizeof($row); $i<sizeof($columns); $i++) {
				domchild($level+3, 'td', '', '');
			}
		}

		domchild($level+3, 'td', '', '');
		domchild($level+4, 'input', array('type' => 'submit',
			 'name' => 'delrow[' . $count . ']',
			 'value' => 'Delete'), '');
		$count++;

	}

	domchild($level+2, 'tr', '', '');

	domchild($level+3, 'td', array('colspan' => sizeof($columns)+2,
		 'class' => 'center'), '');

	domchild($level+4, 'input', array('type' => 'submit',
		 'name' => 'addrow', 'value' => 'Add New'), '');

	domchild($level+4, 'input',
		array('type' => 'submit', 'value' => 'Save'), '');

}

function mkdir_if_not_exist($dir)
{
	if (!file_exists($dir)) {
		if (mkdir($dir) == FALSE) {
			error_msg('Cannot create ' . $dir);
		}
	}

}

function sleep_to_next_second()
{
	$now = microtime(1);
	// cut off the the microseconds and add 1 second
	$next_second = preg_replace('/\..*/', '', $now) + 1;
	$sleep = $next_second - $now;
	usleep($sleep * 1000000);
}

function runs_on_command_line()
{
	if (isset($_SERVER['_']) && preg_match('/php/', $_SERVER['_'])) {
		return TRUE;
	} else {
		return FALSE;
	}
}

function is_ip($ip)
{
	$retval = false;

	// Simple check if value is a ip address
	if (preg_match('/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/',
		  $ip)) {
		$retval = true;
	}

	return $retval;
}

// swap array keys in a two dimensional array
// e.g.
// Array
// (
//    [0] => Array ( [key1] => value1, [key2] => value2, [key3] => value3... )
//    [1] => Array ( [key1] => value1, [key2] => value2, [key3] => value3... )
//    ....
// )
// swaped result
// Array
// (
//     [key1] => Array ( [0] => value1, [1] => value2, [2] => value3.... )
//     [key2] => Array ( [0] => value1, [1] => value2, [2] => value3.... )
//     ....
// )
function array_key_swap($array)
{
	$array_swaped = array();

	foreach ($array as $key1 => $values) {
		foreach ($values as $key2 => $value) {
			$array_swaped[$key2][$key1] = $value;
		}
	}

	return $array_swaped;
}

function filter()
{
	if (isset($_POST['filter'])) {
		if ($_POST['filter']) {
				setFilter(_stripslashes($_POST['filter']));
		} else {
				unset($_SESSION['filter']);
		}
	}

	if (isset($_POST['from'])) {
		setFilterFrom(db_escape_string($_POST['from']));
	}
	if (isset($_POST['to'])) {
		setFilterTo(db_escape_string($_POST['to']));
	}

	// WHERE stamp_inserted >= $from AND stamp_updated <= $end
	/*
	$sqlwhere = ' WHERE stamp_inserted >= \'' . getFilterFrom() .
		'\' AND stamp_updated <= \'' . getFilterTo() . '\'';
	*/

	// TODO
	//$sqlwhere .= filter2sql();
	// for now just add $_POST['filter'] to our sql query
	if (isset($_POST['filter']) && $_POST['filter']) {
		setFilter(getFilter());
	}
	$filter = getFilter();
	if (isset($filter) && $filter) {
		// TODO
		//$sqlwhere .= ' AND ' . $filter;
		$sqlwhere = $filter;
	}

	domchild(3, 'form', array('action' => getSelf(), 'method' => 'post'), '');
	domchild(4, 'div', array('class' => 'center'), '');
	/*
	domchild(5, 'span', '', 'Time interval from:');
	domchild(5, 'input', array('type' => 'text', 'name' => 'from',
		 'value' => _stripslashes(getFilterFrom())), '');
	domchild(5, 'span', '', 'to:');
	domchild(5, 'input', array('type' => 'text', 'name' => 'to',
		 'value' => _stripslashes(getFilterTo())), '');
	domchild(5, 'br', '', '');
	domchild(4, 'div', array('class' => 'center'), '');
	#domchild(5, 'span', '', 'SQL Filter: (SELECT * FROM ' .
	#	 $_SESSION['tablename'] . ' WHERE ');
	*/
	domchild(5, 'span', '', 'SQL Filter: ');
	domchild(5, 'input', array('type' => 'text', 'name' => 'filter',
		 'size' => '30', 'value' => _stripslashes(getFilter())), '');
	#domchild(5, 'span', '', ')');
	domchild(5, 'br', '', '');
	domchild(5, 'input', array('type' => 'submit', 'name' => 'clearfilter',
		 'value' => 'Clear'), '');
	domchild(5, 'input', array('type' => 'submit', 'name' => 'dbquery',
		 'value' => 'Query'), '');

	debug_msg(2, '$sqlwhere = ' . $sqlwhere);

	return($sqlwhere);

}

?>
