<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// our wrapper for pmacct
function pmacct_cmd($opts)
{
	$output = array();
	$pmacct = getPmacctdBinary();

	$output = exec_cmd($pmacct, $opts);

	return $output;
}

// query pmacct
function pmacct_query($type, $primitive, $match, $pipe, $reset_counter)
{
	// first check if the pipe is ok
	if (!pmacct_check_pipe($pipe)) {
		throw new Exception('Cannot connect to pipe ' . $pipe .
				    ' (pmacctd running?)');
	}

	// perform locking of the table
	$opts = '-l ';
	if ($reset_counter == TRUE) {
		
		$opts .= '-r ';
	}
	$opts .= '-n ' . $type . ' -c ' . $primitive . ' -N "' . $match .
		'" -p ' . $pipe;
	$output = pmacct_cmd($opts);

	return($output);
}

// get totals for interface
function pmacct_total()
{
	$retval = array();

	$dev = escapeshellcmd(getInterface());
	$pipe = getPmacctdPipeDir() . '/pmacct_' . $dev . '.pipe';
	$output = pmacct_query('all', 'none', '*', $pipe, TRUE);

	// check if we have only one array element, because the command should
	// only produce one line
	if (sizeof($output) == 1) {
		list($packets, $bytes, $flows, $time) =
						preg_split('/\s+/', $output[0]);
		debug_msg(2, '$packets = ' . $packets);
		debug_msg(2, '$bytes = ' . $bytes);
		debug_msg(2, '$flows = ' . $flows);
		$retval['packets'] = $packets;
		$retval['bytes'] = $bytes;
		$retval['flows'] = $flows;
	} else {
		error_msg('pmacct command produced more lines then expected');
	}

	return($retval);

}

// get rx totals for interface
function pmacct_rxtotal()
{
	$retval = array();

	$dev = escapeshellcmd(getInterface());
	$pipe = getPmacctdPipeDir() . '/pmacct_i_' . $dev . '.pipe';
	$output = pmacct_query('all', 'src_host,dst_host', '*', $pipe, TRUE);

	// check if we have only one array element, because the command should
	// only produce one line
	if (sizeof($output) == 1) {
		list($packets, $bytes, $flows, $time) =
						preg_split('/\s+/', $output[0]);
		debug_msg(2, '$packets = ' . $packets);
		debug_msg(2, '$bytes = ' . $bytes);
		debug_msg(2, '$flows = ' . $flows);
		$retval['packets'] = $packets;
		$retval['bytes'] = $bytes;
		$retval['flows'] = $flows;
	} else {
		error_msg('pmacct command produced more lines then expected');
	}

	return($retval);

}

// get tx totals for interface
function pmacct_txtotal()
{
	$retval = array();

	$dev = escapeshellcmd(getInterface());
	$pipe = getPmacctdPipeDir() . '/pmacct_o_' . $dev . '.pipe';
	$output = pmacct_query('all', 'src_host,dst_host', '*', $pipe, TRUE);

	// check if we have only one array element, because the command should
	// only produce one line
	if (sizeof($output) == 1) {
		list($packets, $bytes, $flows, $time) =
						preg_split('/\s+/', $output[0]);
		debug_msg(2, '$packets = ' . $packets);
		debug_msg(2, '$bytes = ' . $bytes);
		debug_msg(2, '$flows = ' . $flows);
		$retval['packets'] = $packets;
		$retval['bytes'] = $bytes;
		$retval['flows'] = $flows;
	} else {
		error_msg('pmacct command produced more lines then expected');
	}

	return($retval);

}

// get interface counter from pmacct and return array with the counter values
// $retval['dev'] = Interface
// $retval['maxrxrate'] = Maximum interface speed (Resceive)
// $retval['maxtxrate'] = Maximum interface speed (Trasmit)
// $retval['rxbytes'] = Received bytes
// $retval['rxpackets'] = Received packets
// $retval['txbytes'] = Transmited bytes
// $retval['txpackets'] = Transmited packets
// $retval['interval'] = Refresh interval
// $retval['txrate'] = Transmit rate
// $retval['rxrate'] = Receive rate
// $retval['txpercent'] = Percentage transmit rate
// $retval['rxpercent'] = Percentage receive rate
// $retval['last'] = When was the function called (unixtimestemp)
function pmacct_counter()
{
	$dev = getInterface();
	$retval = array('dev' => $dev,
			'maxrate' => getEtherRXSpeed($dev) +
					getEtherTXSpeed($dev),
			'maxrxrate' => getEtherRXSpeed($dev),
			'maxtxrate' => getEtherTXSpeed($dev));
	$regex = '/^\s*' . $dev . ':\s*/i';
	// default interval in seconds
	$interval = 2;
	$retval['interval'] = $interval;

	// pmacct counter
	$rxarray = array();
	$txarray = array();
	$total = pmacct_total();
	$rxtotal = pmacct_rxtotal();
	$txtotal = pmacct_txtotal();
	// total bytes
	$retval['bytes'] = $total['bytes'];
	// total packets
	$retval['packets'] = $total['packets'];
	// total rx
	$retval['rxbytes'] = $rxtotal['bytes'];
	$retval['rxpackets'] = $rxtotal['packets'];
	// total tx
	$retval['txbytes'] = $txtotal['bytes'];
	$retval['txpackets'] = $txtotal['packets'];

	// check if we have icounters in our session
	if (isset($_SESSION['icounters']) && is_array($_SESSION['icounters'])) {
		// calculate rate interval
		$interval = microtime(1) - $_SESSION['icounters']['last'];
		// if the last call of this function is not more then one
		// second ago sleep to the next second
		if ($interval < 1) {
			sleep_to_next_second();
			// save new interval
			$interval = microtime(1) -
				$_SESSION['icounters']['last'];
		}
		// do the rate calculation
		$retval['rate'] = round($retval['bytes'] * 8 / $interval);
		$retval['rxrate'] = round($retval['rxbytes'] * 8 / $interval);
		$retval['txrate'] = round($retval['txbytes'] * 8 / $interval);

		// calculate percentage usage
		$percent = round($retval['rate'] / $retval['maxrate'] * 100);
		if ($retval['maxrxrate'] > 0) {
			$rxpercent = round($retval['rxrate'] /
					   $retval['maxrxrate'] * 100);
		} else {
			$rxpercent = 0;
		}
		if ($retval['maxtxrate'] > 0) {
			$txpercent = round($retval['txrate'] /
					   $retval['maxtxrate'] * 100);
		} else {
			$txpercent = 0;
		}

		if ($percent > 100) {
			$percent = 100;
		}
		if ($rxpercent > 100) {
			$rxpercent = 100;
		}
		if ($txpercent > 100) {
			$txpercent = 100;
		}
		$retval['percent'] = $percent;
		$retval['rxpercent'] = $rxpercent;
		$retval['txpercent'] = $txpercent;
	} else {
		$retval['rate'] = 0;
		$retval['rxrate'] = 0;
		$retval['txrate'] = 0;
		$retval['percent'] = 0;
		$retval['rxpercent'] = 0;
		$retval['txpercent'] = 0;
	}

	// save current unix timestamp
	$retval['last'] = microtime(1);

	// save values in session to do rate calculations
	$_SESSION['icounters'] = $retval;

	return($retval);

}

function pmacct_create_config($level)
{
	$pmacctd_cfgfile = getPmacctdCfgfile();
	$sfacctd_cfgfile = getSfacctdCfgfile();
	$ports_file = dirname($pmacctd_cfgfile) . '/ports';
	$networks_file = dirname($pmacctd_cfgfile) . '/networks';

	$ethers = getEthers();
	$networks = getNetworks();
	$ports = getPorts();

	// first write the ports and networks file because we need it later for
	// pmacctd.conf
	pmacct_create_portlist($level);
	pmacct_create_networklist($level);

	// configuration for pmacctd
	$conf_type = 'pmacct';

	$conf  = "!\n";
	$conf .= "! " . $conf_type . "d.conf\n";
	$conf .= "!\n";
	$conf .= "daemonize: true\n";
	$conf .= "pidfile: /var/run/" . $conf_type . "d.pid\n";
	$conf .= "!\n";
	$conf .= "interface: " . getInterface() . "\n";
	$conf .= "promisc: true\n";
	$conf .= "!\n";
	$conf .= "! storage methods\n";
	$conf .= "plugins: " . getDatabaseType() . "[all], ";
	$conf .= "memory[i_sniff], memory[o_sniff]";
	foreach ($ethers as $ether) {
		$name = preg_replace('/ /', '_',
			strtolower($ether['name']));
		$conf .= ", memory[i_" . $name . "], memory[o_" . $name . "]";
	}
	$conf .= ", memory[i_networks], memory[o_networks]";
	$conf .= "\n";
	$conf .= "plugin_pipe_size: 10240000\n";
	// Disable buffer size for now, because it brakes our realtime graph
	//$conf .= "plugin_buffer_size: 10240\n";
	$conf .= "!\n";
	$conf .= "! sql\n";
	$conf .= "sql_db: " . getDatabaseName() . "\n";
	$conf .= "sql_host: " . getDatabaseHost() . "\n";
	$conf .= "sql_user: " . getDatabaseUser() . "\n";
	$conf .= "sql_passwd: " . getDatabasePassword() . "\n";
	$conf .= "sql_table_version: 5\n";
	$conf .= "sql_refresh_time: 60\n";
	$conf .= "sql_history: 1m\n";
	$conf .= "sql_history_roundoff: m\n";
	$conf .= "sql_dont_try_update: true\n";
	$conf .= "!\n";
	$conf .= "! [all]\n";
	$conf .= "sql_table[all]: acct_v5_%Y%m%d\n";
	$conf .= "aggregate[all]: src_mac,dst_mac,vlan," .
		"src_host,dst_host,src_port,dst_port,tos,proto,flows\n";
	$conf .= "!\n";
	$conf .= "! all traffic on sniff interface\n";
	$conf .= "! incoming\n";
	$conf .= "aggregate[i_sniff]: src_port,dst_port\n";
	if (file_exists($ports_file)) {
		$conf .= "ports_file[i_sniff]: " .
			realpath($ports_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[i_sniff]: 0\n";
	$conf .= "imt_path[i_sniff]: " . getPmacctdPipeDir() .
		"/" . $conf_type . "_i_sniff.pipe\n";
	$conf .= "!\n";
	$conf .= "! all traffic on sniff interface\n";
	$conf .= "! outgoing\n";
	$conf .= "aggregate[o_sniff]: src_port,dst_port\n";
	if (file_exists($ports_file)) {
		$conf .= "ports_file[o_sniff]: " .
			realpath($ports_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[o_sniff]: 0\n";
	$conf .= "imt_path[o_sniff]: " . getPmacctdPipeDir() .
		"/" . $conf_type . "_o_sniff.pipe\n";
	$conf .= "!\n";
	foreach ($ethers as $ether) {
		$name = preg_replace('/ /', '_',
			strtolower($ether['name']));
		$mac = $ether['mac'];
		$agg = 'src_port,dst_port';
		// incoming
		// when name is our local sniff interface we have to use dst
		if ($ether['name'] == getInterface()) {
			$filter = 'ether dst ' . $mac;
		} else {
			$filter = 'ether src ' . $mac;
		}
		$path = getPmacctdPipeDir() . '/' . $conf_type . '_i_' .
			$name . '.pipe';
		$conf .= "! " . $name . "\n";
		$conf .= "! incoming\n";
		$conf .= 'aggregate_filter[i_' . $name . ']: ' . $filter . "\n";
		$conf .= "aggregate[i_" . $name . "]: " . $agg . "\n";
		if (file_exists($ports_file)) {
			$conf .= "ports_file[i_" . $name . "]: " .
				realpath($ports_file) . "\n";
		}
		$conf .= "imt_mem_pools_number[i_" . $name . "]: 0\n";
		$conf .= "imt_path[i_" . $name . "]: " . $path . "\n";
		$conf .= "!\n";
		// outgoing
		// when name is our local sniff interface we have to use src
		if ($ether['name'] == getInterface()) {
			$filter = 'ether src ' . $mac;
		} else {
			$filter = 'ether dst ' . $mac;
		}
		$path = getPmacctdPipeDir() . '/' . $conf_type . '_o_' .
			$name . '.pipe';
		$conf .= "! " . $name . "\n";
		$conf .= "! outgoing\n";
		$conf .= 'aggregate_filter[o_' . $name . ']: ' . $filter . "\n";
		$conf .= "aggregate[o_" . $name . "]: " . $agg . "\n";
		if (file_exists($ports_file)) {
			$conf .= "ports_file[o_" . $name . "]: " .
				realpath($ports_file) . "\n";
		}
		$conf .= "imt_mem_pools_number[o_" . $name . "]: 0\n";
		$conf .= "imt_path[o_" . $name . "]: " . $path . "\n";
		$conf .= "!\n";
	}
	$conf .= "!\n";
	$agg = 'src_net,dst_net';
	// incoming
	$path = getPmacctdPipeDir() . '/' . $conf_type . '_i_networks.pipe';
	$conf .= "! networks\n";
	$conf .= "! incoming\n";
	$conf .= "aggregate[i_networks]: " . $agg . "\n";
	if (file_exists($networks_file)) {
		$conf .= "networks_file[i_networks]: " .
			realpath($networks_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[i_networks]: 0\n";
	$conf .= "imt_path[i_networks]: " . $path . "\n";
	$conf .= "!\n";
	// outgoing
	$path = getPmacctdPipeDir() . '/'. $conf_type . '_o_networks.pipe';
	$conf .= "! networks\n";
	$conf .= "! outgoing\n";
	$conf .= "aggregate[o_networks]: " . $agg . "\n";
	if (file_exists($networks_file)) {
		$conf .= "networks_file[o_networks]: " .
			realpath($networks_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[o_networks]: 0\n";
	$conf .= "imt_path[o_networks]: " . $path . "\n";
	$conf .= "!\n";

	if (can_write($pmacctd_cfgfile)) {
		if ($fd = fopen($pmacctd_cfgfile, 'w')) {
			fwrite($fd, $conf);
			fclose($fd);
			domchild($level, 'h3', '', 'Genereated file: ' .
				realpath($pmacctd_cfgfile));
		} else {
			error_msg('Cannot write to ' . $pmacctd_cfgfile);
		}
	} else {
		error_msg('Cannot write to ' . $pmacctd_cfgfile);
	}

	// configuration for sfacctd
	$conf_type = 'sfacct';

	$conf  = "!\n";
	$conf .= "! " . $conf_type . "d.conf\n";
	$conf .= "!\n";
	$conf .= "daemonize: true\n";
	$conf .= "pidfile: /var/run/" . $conf_type . "d.pid\n";
	$conf .= "!\n";
	$conf .= "sfacctd_renormalize: true\n";
	$conf .= "sfacctd_ext_sampling_rate:2000\n";
	$conf .= "!sfacctd_time_new:true\n";
	$conf .= "!notwendig ?\n";
	$conf .= "!sfacctd_disable_checks:true\n";
	$conf .= "!http://pmacct-discussion.pmacct.narkive.com/E2NcVNSE/sflow";
	$conf .= "-renormalize-not-working-correctly-sql-preprocess-does\n";
	$conf .= "!sql_preprocess: usrf=2000 \n";

	$conf .= "!\n";
	$conf .= "! storage methods\n";
	$conf .= "plugins: " . getDatabaseType() . "[all], ";
	$conf .= "memory[i_sniff], memory[o_sniff]";
	foreach ($ethers as $ether) {
		$name = preg_replace('/ /', '_',
			strtolower($ether['name']));
		$conf .= ", memory[i_" . $name . "], memory[o_" . $name . "]";
	}
	$conf .= ", memory[i_networks], memory[o_networks]";
	$conf .= "\n";
	$conf .= "plugin_pipe_size: 10240000\n";
	// Disable buffer size for now, because it brakes our realtime graph
	//$conf .= "plugin_buffer_size: 10240\n";
	$conf .= "!\n";
	$conf .= "! sql\n";
	$conf .= "sql_db: " . getDatabaseName() . "\n";
	$conf .= "sql_host: " . getDatabaseHost() . "\n";
	$conf .= "sql_user: " . getDatabaseUser() . "\n";
	$conf .= "sql_passwd: " . getDatabasePassword() . "\n";
	$conf .= "sql_table_version: 5\n";
	$conf .= "sql_refresh_time: 60\n";
	$conf .= "sql_history: 1m\n";
	$conf .= "sql_history_roundoff: m\n";
	$conf .= "!\n";
	$conf .= "! [all]\n";
	$conf .= "sql_table[all]: acct_v5_%Y%m%d\n";
	$conf .= "aggregate[all]: src_mac,dst_mac,vlan," .
		"src_host,dst_host,src_port,dst_port,tos,proto,flows\n";
	$conf .= "!\n";
	$conf .= "! all traffic on sniff interface\n";
	$conf .= "! incoming\n";
	$conf .= "aggregate[i_sniff]: src_port,dst_port\n";
	if (file_exists($ports_file)) {
		$conf .= "ports_file[i_sniff]: " .
			realpath($ports_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[i_sniff]: 0\n";
	$conf .= "imt_path[i_sniff]: " . getPmacctdPipeDir() .
		"/" . $conf_type . "_i_sniff.pipe\n";
	$conf .= "!\n";
	$conf .= "! all traffic on sniff interface\n";
	$conf .= "! outgoing\n";
	$conf .= "aggregate[o_sniff]: src_port,dst_port\n";
	if (file_exists($ports_file)) {
		$conf .= "ports_file[o_sniff]: " .
			realpath($ports_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[o_sniff]: 0\n";
	$conf .= "imt_path[o_sniff]: " . getPmacctdPipeDir() .
		"/" . $conf_type . "_o_sniff.pipe\n";
	$conf .= "!\n";
	foreach ($ethers as $ether) {
		$name = preg_replace('/ /', '_',
			strtolower($ether['name']));
		$mac = $ether['mac'];
		$agg = 'src_port,dst_port';
		// incoming
		// when name is our local sniff interface we have to use dst
		if ($ether['name'] == getInterface()) {
			$filter = 'ether dst ' . $mac;
		} else {
			$filter = 'ether src ' . $mac;
		}
		$path = getPmacctdPipeDir() . '/' . $conf_type . '_i_' .
			$name . '.pipe';
		$conf .= "! " . $name . "\n";
		$conf .= "! incoming\n";
		$conf .= 'aggregate_filter[i_' . $name . ']: ' . $filter . "\n";
		$conf .= "aggregate[i_" . $name . "]: " . $agg . "\n";
		if (file_exists($ports_file)) {
			$conf .= "ports_file[i_" . $name . "]: " .
				realpath($ports_file) . "\n";
		}
		$conf .= "imt_mem_pools_number[i_" . $name . "]: 0\n";
		$conf .= "imt_path[i_" . $name . "]: " . $path . "\n";
		$conf .= "!\n";
		// outgoing
		// when name is our local sniff interface we have to use src
		if ($ether['name'] == getInterface()) {
			$filter = 'ether src ' . $mac;
		} else {
			$filter = 'ether dst ' . $mac;
		}
		$path = getPmacctdPipeDir() . '/' . $conf_type . '_o_' .
			$name . '.pipe';
		$conf .= "! " . $name . "\n";
		$conf .= "! outgoing\n";
		$conf .= 'aggregate_filter[o_' . $name . ']: ' . $filter . "\n";
		$conf .= "aggregate[o_" . $name . "]: " . $agg . "\n";
		if (file_exists($ports_file)) {
			$conf .= "ports_file[o_" . $name . "]: " .
				realpath($ports_file) . "\n";
		}
		$conf .= "imt_mem_pools_number[o_" . $name . "]: 0\n";
		$conf .= "imt_path[o_" . $name . "]: " . $path . "\n";
		$conf .= "!\n";
	}
	$conf .= "!\n";
	$agg = 'src_net,dst_net';
	// incoming
	$path = getPmacctdPipeDir() . '/' . $conf_type . '_i_networks.pipe';
	$conf .= "! networks\n";
	$conf .= "! incoming\n";
	$conf .= "aggregate[i_networks]: " . $agg . "\n";
	if (file_exists($networks_file)) {
		$conf .= "networks_file[i_networks]: " .
			realpath($networks_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[i_networks]: 0\n";
	$conf .= "imt_path[i_networks]: " . $path . "\n";
	$conf .= "!\n";
	// outgoing
	$path = getPmacctdPipeDir() . '/'. $conf_type . '_o_networks.pipe';
	$conf .= "! networks\n";
	$conf .= "! outgoing\n";
	$conf .= "aggregate[o_networks]: " . $agg . "\n";
	if (file_exists($networks_file)) {
		$conf .= "networks_file[o_networks]: " .
			realpath($networks_file) . "\n";
	}
	$conf .= "imt_mem_pools_number[o_networks]: 0\n";
	$conf .= "imt_path[o_networks]: " . $path . "\n";
	$conf .= "!\n";

	if (can_write($sfacctd_cfgfile)) {
		if ($fd = fopen($sfacctd_cfgfile, 'w')) {
			fwrite($fd, $conf);
			fclose($fd);
			domchild($level, 'h3', '', 'Genereated file: ' .
				realpath($sfacctd_cfgfile));
		} else {
			error_msg('Cannot write to ' . $sfacctd_cfgfile);
		}
	} else {
		error_msg('Cannot write to ' . $sfacctd_cfgfile);
	}

	pmacct_restart($level);

}

function pmacct_create_portlist($level)
{
	$pmacctd_cfgfile = getPmacctdCfgfile();
	$ports_file = dirname($pmacctd_cfgfile) . '/ports';

	$ports = getPorts();

	$portlist = "! monitored ports\n";
	foreach ($ports as $ports) {
		$portlist .= $ports['port'] . "\n";
	}

	if (can_write($ports_file)) {
		if ($fd = fopen($ports_file, 'w')) {
			fwrite($fd, $portlist);
			fclose($fd);
			domchild($level, 'h3', '', 'Genereated file: ' .
				realpath($ports_file));
		} else {
			error_msg('Cannot write to ' . $ports_file);
		}
	} else {
		error_msg('Cannot write to ' . $ports_file);
	}

}

function pmacct_create_networklist($level)
{
	$pmacctd_cfgfile = getPmacctdCfgfile();
	$networks_file = dirname($pmacctd_cfgfile) . '/networks';

	$networks = getNetworks();

	$networklist = "! monitored networks\n";
	foreach ($networks as $network) {
		$networklist .= $network['net'] . "\n";
	}

	if (can_write($networks_file)) {
		if ($fd = fopen($networks_file, 'w')) {
			fwrite($fd, $networklist);
			fclose($fd);
			domchild($level, 'h3', '', 'Genereated file: ' .
				realpath($networks_file));
		} else {
			error_msg('Cannot write to ' . $networks_file);
		}
	} else {
		error_msg('Cannot write to ' . $networks_file);
	}

}

// check if we can connect to the pmacct pipe
function pmacct_check_pipe($pipe)
{
	$ret = FALSE;
	$pmacct = getPmacctdBinary();
	if (exec($pmacct . ' -s -p ' . $pipe)) {
		$ret = TRUE;
	}
	return $ret;
}


function pmacct_restart($level)
{

	domchild($level, 'h3', '', 'Please restart pmacctd');
	//exec_cmd('sudo', '/etc/init.d/pmacct restart');

}

?>
