<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// our wrapper for the rrdtool
function rrdtool_cmd($opts)
{
	$output = array();
	$rrdtool = getRrdtoolBinary();

	$output = exec_cmd($rrdtool, $opts);

	return($output);
}

// create rrdtool database
function rrdtool_create_db($dbfile, $dsname)
{
	// Specs for the RRD Database
	$rrd_ds = 'DS:' . $dsname . ':ABSOLUTE:90:0:U';
	$rrd_rra = 'RRA:AVERAGE:0.5:1:3600 ' .
		   'RRA:AVERAGE:0.5:60:10080 ' .
		   'RRA:AVERAGE:0.5:3600:8760 ' .
		   'RRA:AVERAGE:0.5:86400:3650';
	// Update interval
	$rrd_interval = '1';

	$opts = 'create ' . $dbfile . ' -s ' . $rrd_interval . ' ' . $rrd_ds .
		' ' . $rrd_rra;

	rrdtool_cmd($opts);
}

// update rrdtool database
function rrdtool_update_db($dbfile, $timestamp, $counter)
{
	$opts = 'update ' . $dbfile . ' ' . $timestamp . ':' . $counter;
	rrdtool_cmd($opts);
}

// get last timestamp from database and return the timestamp
function rrdtool_last_dbentry($dbfile)
{
	$opts = 'last ' . $dbfile;
	$last = rrdtool_cmd($opts);
	// check if last is a timestamp
	if (!preg_match('/^\d+$/', $last[0])) {
		error_msg('Cannot get last timestamp from database');
	}
	return $last[0];
}

// query rrdtool database and return sum value for timestamp range
function rrdtool_query_db($dbfile, $from, $to)
{
	$ret = 0;
	// we have to subtract 1 to get correct results for timestamp
	$fixedfrom = $from - 1;
	$fixedto = $to - 1;
	$opts = 'fetch '. $dbfile . ' AVERAGE -s ' . $fixedfrom .
		' -e ' . $fixedto;
	$output = rrdtool_cmd($opts);

	foreach ($output as $line) {
		$match = '/^\d+:\s*/';
		// check if line starts with the unix timestamp
		if (preg_match($match, $line)) {
			// cut off timestamp, because we only need the value
			$result = preg_replace($match, '', $line);
			// if we have nan as value then we have no data for
			// that timestamp and just use 0
			if (preg_match('/^nan$/', $result)) {
				debug_msg(1, 'found nan');
				$result = 0;
			// check if we have a ',' in the line and replace
			// it with '.' (some locales use ',' as separator for
			// float values)
			} else if (preg_match('/,/', $result)) {
				$ret += floatval(preg_replace('/,/', '.',
						 $result));
			} else {
				$ret += floatval($result);
			}
		}
	}

	return $ret;
}

// get counters from rrd databases in getRrdtoolDbDir()/$name/
// (i_total.rrd and o_total.rrd) files will be used
// $retval['maxrxrate'] = Maximum speed (Resceive)
// $retval['maxtxrate'] = Maximum speed (Transmit)
// $retval['maxrate'] = Maximum speed (Resceive+Transmit)
// $retval['interval'] = Refresh interval
// $retval['lastin'] = When was the database last updated (unixtimestemp)
// $retval['lastout'] = When was the database last updated (unixtimestemp)
// $retval['rxbytes'] = Received bytes
// $retval['txbytes'] = Transmited bytes
// $retval['bytes'] = Total bytes (rxbytes+txbytes)
// $retval['txrate'] = Transmit rate
// $retval['rxrate'] = Receive rate
// $retval['rate'] = Total rate (txrate+rxrate)
// $retval['txpercent'] = Percentage transmit rate
// $retval['rxpercent'] = Percentage receive rate
// $retval['percent'] = Total Percentage
// $retval['last'] = When was the function called (unixtimestemp)
function rrdtool_counter($name)
{
	$retval = array();
	$db_base = getRrdtoolDbDir() . '/' . $name;
	// default interval in seconds
	$retval['interval'] = 2;
	// get rates from config
	$retval['maxrxrate'] = getEtherRXSpeed($name);
	$retval['maxtxrate'] = getEtherTXSpeed($name);
	$retval['maxrate'] = getEtherRXSpeed($name) + getEtherTXSpeed($name);
	// how many entries to take from database for calculations
	$entries = 5;

	// check where the db files are
	if (file_exists($db_base . '/i_total.rrd')) {
		$dbin = $db_base . '/i_total.rrd';
	} else if (file_exists($db_base . '/ports/i_total.rrd')) {
		$dbin = $db_base . '/ports/i_total.rrd';
	} else {
		throw new Exception('Cannot find database file i_total.rrd');
	}
	if (file_exists($db_base . '/o_total.rrd')) {
		$dbout = $db_base . '/o_total.rrd';
	} else if (file_exists($db_base . '/ports/o_total.rrd')) {
		$dbout = $db_base . '/ports/o_total.rrd';
	} else {
		throw new Exception('Cannot find database file o_total.rrd');
	}

	if (isset($_SESSION[$name]['last']) && $_SESSION[$name]['last']) {
		$diff = microtime(1) - $_SESSION[$name]['last'];
		// if the last call of this function is not more then one
		// second ago sleep to the next second
		if ($diff <  1) {
			sleep_to_next_second();
		}
	}

	// we have to know when the database was last updated
	$retval['lastin'] = rrdtool_last_dbentry($dbin);
	$retval['lastout'] = rrdtool_last_dbentry($dbout);

	// if last entry of database is 10 seconds away from now set every
	// counter to -1 and leave function (this can happen when there
	// is a problem the pmaact or the update of the rrd database)
	if ($retval['lastin'] < round(microtime(1) - 10)) {
		$retval['rxbytes'] = -1;
		$retval['txbytes'] = -1;
		$retval['bytes'] = -1;
		$retval['rxrate'] = -1;
		$retval['txrate'] = -1;
		$retval['rate'] = -1;
		$retval['rxpercent'] = -1;
		$retval['txpercent'] = -1;
		$retval['percent'] = -1;
		// save current unix timestamp to know when this function was
		// called
		$retval['last'] = microtime(1);
		// commit this value to session
		$_SESSION[$name]['last'] = $retval['last'];
		return($retval);
	}

	// add one to really only get $entries from database
	$from = $retval['lastin'] - $entries + 1;
	$to = $retval['lastin'];
	// this value is used for later rate calculation
	$rate_rxbytes = rrdtool_query_db($dbin, $from, $to);

	// add one to really only get $entries from database
	$from = $retval['lastin'] - $entries + 1;
	$to = $retval['lastin'];
	// this value is used for later rate calculation
	$rate_txbytes = rrdtool_query_db($dbout, $from, $to);

	// for return rxbytes and txbytes we only want the last entry from db
	$retval['rxbytes'] = round(rrdtool_query_db($dbin, $to, $to));
	$retval['txbytes'] = round(rrdtool_query_db($dbout, $to, $to));
	$retval['bytes'] = $retval['rxbytes'] + $retval['txbytes'];

	// do the rate calculation
	$retval['rxrate'] = round($rate_rxbytes * 8 / $entries);
	$retval['txrate'] = round($rate_txbytes * 8 / $entries);
	$retval['rate'] = $retval['rxrate'] + $retval['txrate'];

	// rxpercent
	if ($retval['maxrxrate'] > 0) {
		$rxpercent = round($retval['rxrate'] /
				   $retval['maxrxrate'] * 100);
	} else {
		$rxpercent = 0;
	}
	// txpercent
	if ($retval['maxtxrate'] > 0) {
		$txpercent = round($retval['txrate'] /
				   $retval['maxtxrate'] * 100);
	} else {
		$txpercent = 0;
	}
	// calculate percentage usage
	$percent = round($retval['rate'] / $retval['maxrate'] * 100);

	// when the maxrate is wrongly configured we can have
	// values > 100 percent. for now we just set them to 100
	// percent
	if ($rxpercent > 100) {
		$rxpercent = 100;
	}
	if ($txpercent > 100) {
		$txpercent = 100;
	}
	if ($percent > 100) {
		$percent = 100;
	}
	$retval['rxpercent'] = $rxpercent;
	$retval['txpercent'] = $txpercent;
	$retval['percent'] = $percent;

	// save current unix timestamp to know when this function was called
	$retval['last'] = microtime(1);
	// commit this value to session
	$_SESSION[$name]['last'] = $retval['last'];

	return($retval);
}

// rrdtool grapher function
function rrdtool_graph($file, $options)
{
	$tmpfile = tempnam('/tmp', 'rrd');
	debug_msg(2, '$tmpfile = ' . $tmpfile);

	// first write the output to a temporary file
	$opts = 'graph ' . $tmpfile . ' -a PNG ' . $options;
	rrdtool_cmd($opts);

	// Read and write for owner, read for everybody else
	chmod($tmpfile, 0644);

	// now move file to the correct location, if this failes remove the
	// temporary file
	if (!rename($tmpfile, $file)) {
		unlink($tmpfile);
	}

}

// update all databases
function rrdtool_update_all_dbs()
{
	rrdtool_update_all_port_dbs();
	rrdtool_update_network_dbs();
}

// update port databases
// functions return counter values
function rrdtool_update_all_port_dbs()
{
	// For each ether entry we have our own subdirectory and databases
	foreach (getEthers() as $ether) {
		rrdtool_update_port_dbs(strtolower($ether['name']));
	}
	// Also update the total sniff db
	rrdtool_update_port_dbs('sniff');
}

// update port database for a specific ether name
function rrdtool_update_port_dbs($name)
{
	$match = '';
	$primitive = 'src_port,dst_port';

	// Save port values for later pmacct query
	foreach (getPorts() as $port) {
		$match .= $port['port'] . ',*;';
		$match .= '*,' . $port['port'] . ';';
	}
	// Total value for query
	$match .= '0,0';

	$etherpath = getRrdtoolDbDir() . '/' . $name;
	$portpath = $etherpath . '/ports';
	$dsname = 'total';
	// Create rrdtool db sub directories if they do not exist
	mkdir_if_not_exist($etherpath);
	mkdir_if_not_exist($portpath);
	if (!file_exists($portpath . '/i_total.rrd')) {
		$totalfile = $portpath . '/i_total.rrd';
		// Create rrd databases for the total counter (incoming)
		rrdtool_create_db($totalfile, $dsname);
	}
	if (!file_exists($portpath . '/o_total.rrd')) {
		$totalfile = $portpath . '/o_total.rrd';
		// Create rrd databases for the total counter (outgoing)
		rrdtool_create_db($totalfile, $dsname);
	}
	foreach (getPorts() as $port) {
		$portfile = $portpath . '/i_' . $port['port'] . '.rrd';
		$dsname = $port['port'];
		if (!file_exists($portfile)) {
			// Create rrd database for port (incoming)
			rrdtool_create_db($portfile, $dsname);
		}
		$portfile = $portpath . '/o_' . $port['port'] . '.rrd';
		if (!file_exists($portfile)) {
			// Create rrd database for port (outgoing)
			rrdtool_create_db($portfile, $dsname);
		}
	}

	$lockfile = $etherpath . '/.LCK';
	// check if there is already a LOCK for that name
	if (file_exists($lockfile)) {
		$stat = stat($lockfile);
		// if the LOCK file is older then 5 minutes then update the
		// mtime
		if ($stat['mtime']+300 < round(microtime(1))) {
			debug_msg(1, 'updating LOCK for ' . $name);
			touch($lockfile);
		} else {
			debug_msg(1, 'another process tries to write to ' .
				  $name . ' databases');
			return;
		}
	// if there is no LOCK then create the file
	} else {
		debug_msg(1, 'creating LOCK for ' . $name);
		touch($lockfile);
	}

	// because the rrd database can only save the unix timestamp in
	// seconds without microseconds we have to sleep to the exact next
	// second to have an exact timestamp in the rrd database
	sleep_to_next_second();

	// updates to the rrd database are only allowed once per second,
	// so we have to get the last timestamp from one of the
	// databases and compare it to the current timestamp
	// if we already have data for that second in the database just
	// leave the function
	$dbfile = $portpath . '/i_total.rrd';
	$last = rrdtool_last_dbentry($dbfile);
	if ($last == round(microtime(1))) {
		debug_msg(1, $dbfile . ' has already data for that timestamp ('
			  . $last . ')');
		sleep_to_next_second();
		return;
	}

	// save current timestamp (rounded to seconds)
	$now = round(microtime(1));

	// get incoming counter from pmacct
	$pipe = getPmacctdPipeDir() . '/pmacct_i_' . $name .
			'.pipe';
	$icounter = pmacct_query('all', $primitive, $match, $pipe,
				  TRUE);

	// get outgoing counter from pmacct
	$pipe = getPmacctdPipeDir() . '/pmacct_o_' . $name .
			'.pipe';
	$ocounter = pmacct_query('all', $primitive, $match, $pipe,
				  TRUE);

	$itotal = 0;
	$itotalp = 0;
	// total incoming
	foreach ($icounter as $icount) {
		list($packets, $bytes, $flows, $time) =
			preg_split('/\s+/', $icount);
		$itotal += $bytes;
		$itotalp += $packets;
	}

	$ototal = 0;
	$ototalp = 0;
	// total outgoing
	foreach ($ocounter as $ocount) {
		list($packets, $bytes, $flows, $time) =
			preg_split('/\s+/', $ocount);
		$ototal += $bytes;
		$ototalp += $packets;
	}

	// update rrd databases for each port
	foreach (getPorts() as $port) {
		// incoming
		list($packets, $bytes, $flows, $time) =
			preg_split('/\s+/', array_shift($icounter));
		$p_itotal = $bytes;
		list($packets, $bytes, $flows, $time) =
			preg_split('/\s+/', array_shift($icounter));
		$p_itotal += $bytes;
		$idbfile = $portpath . '/i_' . $port['port'] . '.rrd';
		rrdtool_update_db($idbfile, $now, $p_itotal);

		// outgoing
		list($packets, $bytes, $flows, $time) =
			preg_split('/\s+/', array_shift($ocounter));
		$p_ototal = $bytes;
		list($packets, $bytes, $flows, $time) =
			preg_split('/\s+/', array_shift($ocounter));
		$p_ototal += $bytes;
		$odbfile = $portpath . '/o_' . $port['port'] . '.rrd';
		rrdtool_update_db($odbfile, $now, $p_ototal);
	}

	// update also the total rrd databases
	$dbfile = $portpath . '/i_total.rrd';
	rrdtool_update_db($dbfile, $now, $itotal);
	$dbfile = $portpath . '/o_total.rrd';
	rrdtool_update_db($dbfile, $now, $ototal);

	if (file_exists($lockfile)) {
		debug_msg(1, 'removing LOCK for ' . $name);
		unlink($lockfile);
	}

}

// update network databases
// functions return counter values
function rrdtool_update_network_dbs()
{
	$ret = array();
	$imatch = '';
	$omatch = '';
	$primitive = 'src_net,dst_net';

	// Save port values for later pmacct query
	foreach (getNetworks() as $network) {
		// we need to cut off the network mask
		$net = preg_replace('#/.*$#', '', $network['net']);
		$imatch .= '*,' . $net . ';';
		$omatch .= $net . ',*;';
	}
	// Total value for query
	$imatch .= '0,0';
	$omatch .= '0,0';

	$networkpath = getRrdtoolDbDir() . '/networks';
	$dsname = 'total';
	// Create rrdtool db sub directories if they do not exist
	mkdir_if_not_exist($networkpath);
	if (!file_exists($networkpath . '/i_total.rrd')) {
		$totalfile = $networkpath . '/i_total.rrd';
		// Create rrd databases for the total counter (incoming)
		rrdtool_create_db($totalfile, $dsname);
	}
	if (!file_exists($networkpath . '/o_total.rrd')) {
		$totalfile = $networkpath . '/o_total.rrd';
		// Create rrd databases for the total counter (outgoing)
		rrdtool_create_db($totalfile, $dsname);
	}
	foreach (getNetworks() as $network) {
		$net = preg_replace('#/.*$#', '', $network['net']);
		// convert colons to underscore
		$networkfile = $networkpath . '/i_' .
			preg_replace('/:/', '_', $net) . '.rrd';
		// dots/colons are not allowed as ds so convert them to
		// underscore
		$dsname = preg_replace('/(\.|:+)/', '_', $net);
		if (!file_exists($networkfile)) {
			// Create rrd database for port (incoming)
			rrdtool_create_db($networkfile, $dsname);
		}
		// convert colons to underscore
		$networkfile = $networkpath . '/o_' .
			preg_replace('/:/', '_', $net) . '.rrd';
		if (!file_exists($networkfile)) {
			// Create rrd database for port (outgoing)
			rrdtool_create_db($networkfile, $dsname);
		}
	}

	// updates to the rrd database are only allowed once per second,
	// so we have to get the last timestamp from one of the
	// databases and compare it to the current timestamp
	// if we already have data for that second in the database just
	// leave the function
	$dbfile = $networkpath . '/i_total.rrd';
	$last = rrdtool_last_dbentry($dbfile);
	if ($last == round(microtime(1))) {
		debug_msg(1, $dbfile . ' has already data for that timestamp ('
			  . $last . ')');
		sleep_to_next_second();
		return;
	}

	// because the rrd database can only save the unix timestamp in
	// seconds without microseconds we have to sleep to the exact next
	// second to have an exact timestamp in the rrd database
	sleep_to_next_second();

	// save current timestamp (rounded to seconds)
	$now = round(microtime(1));

	$pipe = getPmacctdPipeDir() . '/pmacct_i_networks.pipe';
	// get incoming counter from pmacct
	$icounter = pmacct_query('bytes', $primitive, $imatch, $pipe, TRUE);

	$pipe = getPmacctdPipeDir() . '/pmacct_o_networks.pipe';
	// get outgoing counter from pmacct
	$ocounter = pmacct_query('bytes', $primitive, $omatch, $pipe, TRUE);

	$itotal = 0;
	// total incoming
	foreach ($icounter as $icount) {
		$itotal += $icount;
	}
	$ret['network']['i_total'] = $itotal;

	$ototal = 0;
	// total outgoing
	foreach ($ocounter as $ocount) {
		$ototal += $ocount;
	}
	$ret['network']['o_total'] = $ototal;

	// update rrd databases for each network
	foreach (getNetworks() as $network) {
		$net = preg_replace('#/.*$#', '', $network['net']);
		// incoming
		$n_itotal = array_shift($icounter);
		// convert colons to underscore
		$idbfile = $networkpath . '/i_' .
			preg_replace('/:/', '_', $net) . '.rrd';
		rrdtool_update_db($idbfile, $now, $n_itotal);
		$ret['network']['i_' . $net] = $n_itotal;

		// outgoing
		$n_ototal = array_shift($ocounter);
		// convert colons to underscore
		$odbfile = $networkpath . '/o_' .
			preg_replace('/:/', '_', $net) . '.rrd';
		rrdtool_update_db($odbfile, $now, $n_ototal);
		$ret['network']['o_' . $net] = $n_ototal;
	}

	// update also the total rrd databases
	$dbfile = $networkpath . '/i_total.rrd';
	rrdtool_update_db($dbfile, $now, $itotal);
	$dbfile = $networkpath . '/o_total.rrd';
	rrdtool_update_db($dbfile, $now, $ototal);

	return $ret;
}

// function to generate png graphics with rrdtool
function rrdtool_generate_graphics()
{
	rrdtool_generate_all_ether_graphics();
	rrdtool_generate_network_graphics();
}

// function to generate all ether png graphics with rrdtool
function rrdtool_generate_all_ether_graphics()
{
	foreach (getEthers() as $ether) {
		rrdtool_generate_ether_graphics($ether['name']);
	}
	// Also update the total sniff graphics
	rrdtool_generate_ether_graphics('sniff');
}

// function to generate ether png graphics with rrdtool
function rrdtool_generate_ether_graphics($name)
{
	$first = TRUE;
	$stack = '';

	$etherpath = getRrdtoolDbDir() . '/' . $name;
	$portpath = $etherpath . '/ports';

	// Generate options for graphs
	// total
	$in_db = $portpath . '/i_total.rrd';
	$out_db = $portpath . '/o_total.rrd';
	if (!file_exists($in_db)) {
		error_msg($in_db . 'No such file');
	}
	if (!file_exists($out_db)) {
		error_msg($out_db . 'No such file');
	}

	$DEF  = '"DEF:i_total=' . $in_db . ':total:AVERAGE" ';
	$DEF .= '"DEF:o_total=' . $out_db . ':total:AVERAGE" ';
	// convert bytes to bits
	$CDEF  = '"CDEF:in=i_total,-8,*" ';
	$CDEF .= '"CDEF:in2=i_total,8,*" ';
	$CDEF .= '"CDEF:out=o_total,8,*" ';
	$LINE  = '"LINE1:in#000099:total" ';
	$LINE .= '"LINE1:out#000099" ';
        $GPRINT  = 'COMMENT:"\\n" ';
        $GPRINT .= 'GPRINT:in2:MIN:"IN  Min %6.2lf %s" ';
        $GPRINT .= 'GPRINT:in2:AVERAGE:"IN  Avg %6.2lf %s" ';
        $GPRINT .= 'GPRINT:in2:MAX:"IN  Max %6.2lf %s" ';
        //$GPRINT .= 'COMMENT:"\\n" ';
        $GPRINT .= 'GPRINT:out:MIN:"OUT Min %6.2lf %s" ';
        $GPRINT .= 'GPRINT:out:AVERAGE:"OUT Avg %6.2lf %s" ';
        $GPRINT .= 'GPRINT:out:MAX:"OUT Max %6.2lf %s" ';
	$IAREA  = '';
	$OAREA  = '';
	// ports
	foreach (getPorts() as $port) {
		$p_nr = $port['port'];
		$color = $port['color'];
		$portname = $port['name'];
		// incoming
		$in = 'i_' . $p_nr;
		$in_db = $portpath . '/i_' . $p_nr . '.rrd';
		if (!file_exists($in_db)) {
			error_msg($in_db . 'No such file');
		}
		$DEF .= '"DEF:' . $in . '=' . $in_db . ':' . $p_nr .
				':AVERAGE" ';
		// convert bytes to bits
		$CDEF .= '"CDEF:in_' . $p_nr . '=' . $in . ',-8,*" ';
		$IAREA .= '"AREA:in_' . $p_nr . '#' . $color . ':' .
				$portname . $stack . '" ';
		// outgoing
		$out = 'o_' . $p_nr;
		$out_db = $portpath . '/o_' . $p_nr . '.rrd';
		if (!file_exists($out_db)) {
			error_msg($out_db . 'No such file');
		}
		$DEF .= '"DEF:' . $out . '=' . $out_db . ':' .
				$p_nr .  ':AVERAGE" ';
		// convert bytes to bits
		$CDEF .= '"CDEF:out_' . $p_nr . '=' . $out . ',8,*" ';
		$OAREA .= '"AREA:out_' . $p_nr . '#' . $color .
				':' . $stack . '" ';
		if ($first == TRUE) {
			$stack = ':STACK';
			$first = FALSE;
		}
	}
	// Add zero line
	$LINE .= '"LINE1:0#000000:" ';

	// titel option
	$titel = ' -t ' . $name . ' ';

	// vertical label option
	$vlabel = ' -v "in <-- traffic/(bits/s) --> out" ';

	// options for preview graph
	$opts_preview = $titel . ' ' . $DEF . ' ' . $CDEF . ' ' . $IAREA . ' ' .
			$OAREA . ' ' . $LINE;

	// options for all other graphs
	$opts = $titel . $vlabel . ' ' . $DEF . ' ' . $CDEF . ' ' . $IAREA .
			' ' .  $OAREA . ' ' . $LINE . ' ' . $GPRINT;

	graph_halfday_preview($name, $opts_preview);

	graph_halfday($name, $opts);

	graph_one_week($name, $opts);

	graph_one_month($name, $opts);

	graph_one_year($name, $opts);

}

// function to generate network png graphics with rrdtool
function rrdtool_generate_network_graphics()
{

	$networkpath = getRrdtoolDbDir() . '/networks';
	$first = true;
	$stack = '';

	// Generate options for graphs
	// total
	$in_db = $networkpath . '/i_total.rrd';
	$out_db = $networkpath . '/o_total.rrd';
	if (!file_exists($in_db)) {
		error_msg($in_db . 'No such file');
	}
	if (!file_exists($out_db)) {
		error_msg($out_db . 'No such file');
	}

	$DEF  = '"DEF:i_total=' . $in_db . ':total:AVERAGE" ';
	$DEF .= '"DEF:o_total=' . $out_db . ':total:AVERAGE" ';
	// convert bytes to bits
	$CDEF  = '"CDEF:in=i_total,-8,*" ';
	$CDEF .= '"CDEF:in2=i_total,8,*" ';
	$CDEF .= '"CDEF:out=o_total,8,*" ';
	$LINE  = '"LINE1:in#000099:total" ';
	$LINE .= '"LINE1:out#000099" ';
        $GPRINT  = 'COMMENT:"\\n" ';
        $GPRINT .= 'GPRINT:in2:MIN:"IN  Min %6.2lf %s" ';
        $GPRINT .= 'GPRINT:in2:AVERAGE:"IN  Avg %6.2lf %s" ';
        $GPRINT .= 'GPRINT:in2:MAX:"IN  Max %6.2lf %s" ';
        //$GPRINT .= 'COMMENT:"\\n" ';
        $GPRINT .= 'GPRINT:out:MIN:"OUT Min %6.2lf %s" ';
        $GPRINT .= 'GPRINT:out:AVERAGE:"OUT Avg %6.2lf %s" ';
        $GPRINT .= 'GPRINT:out:MAX:"OUT Max %6.2lf %s" ';
	$IAREA  = '';
	$OAREA  = '';
	// networks
	foreach (getNetworks() as $network) {
		// we need to cut off the network mask
		// TODO handle ":" with IPv6 correctly. For now just convert
		// them to underscore
		$netfull = preg_replace('/:/', '_', $network['net']);
		$net = preg_replace('#/.*$#', '', $network['net']);
		$color = $network['color'];
		$netname = $network['name'];
		// dots/colons are not allowed as ds so convert them to
		// underscore
		$dsname = preg_replace('/(\.|:+)/', '_', $net);
		// incoming
		$in = 'i_' . $dsname;
		// convert colons to underscore
		$in_db = $networkpath . '/i_' .
			preg_replace('/:/', '_', $net) . '.rrd';
		if (!file_exists($in_db)) {
			error_msg($in_db . 'No such file');
		}
		$DEF .= '"DEF:' . $in . '=' . $in_db . ':' . $dsname .
				':AVERAGE" ';
		// convert bytes to bits
		$CDEF .= '"CDEF:in_' . $dsname . '=' . $in . ',-8,*" ';
		$IAREA .= '"AREA:in_' . $dsname . '#' . $color . ':' .
			$netname . ' (' . $netfull . ')' . $stack . '" ';
		// outgoing
		$out = 'o_' . $dsname;
		// convert colons to underscore
		$out_db = $networkpath . '/o_' .
			preg_replace('/:/', '_', $net) . '.rrd';
		if (!file_exists($out_db)) {
			error_msg($out_db . 'No such file');
		}
		$DEF .= '"DEF:' . $out . '=' . $out_db . ':' .
				$dsname .  ':AVERAGE" ';
		// convert bytes to bits
		$CDEF .= '"CDEF:out_' . $dsname . '=' . $out . ',8,*" ';
		$OAREA .= '"AREA:out_' . $dsname . '#' . $color . ':' .
			$stack . '" ';
		if ($first == TRUE) {
			$stack = ':STACK';
			$first = FALSE;
		}
	}
	// Add zero line
	$LINE .= '"LINE1:0#000000:" ';

	// titel option
	$titel = ' -t Network ';

	// vertical label option
	$vlabel = ' -v "to <-- traffic/(bits/s) --> from" ';

	// options for preview graph
	$opts_preview = $titel . ' ' . $DEF . ' ' . $CDEF . ' ' . $IAREA . ' ' .
			$OAREA . ' ' . $LINE;

	// options for all other graphs
	$opts = $titel . $vlabel . ' ' . $DEF . ' ' . $CDEF . ' ' . $IAREA .
			' ' .  $OAREA . ' ' . $LINE . ' ' . $GPRINT;

	// create the graphs

	graph_halfday_preview('networks', $opts_preview);

	graph_halfday('networks', $opts);

	graph_one_week('networks', $opts);

	graph_one_month('networks', $opts);

	graph_one_year('networks', $opts);
}

function graph_halfday_preview($name, $opts)
{
	// Clears file status cache
	clearstatcache();

	// halfday (preview)
	$final_opts = ' -g -w 400 -h 150 -s end-12h -S 120 ' . $opts;
	$graph = getRrdtoolGraphDir() . '/' . $name . '_12h_small.png';
	// check if the graph already exist, when not create it
	if (file_exists($graph)) {
		// only update graph when old version is older then 10 seconds
		if (time()-filemtime($graph) > 10) {
			rrdtool_graph($graph, $final_opts);
		}
	} else {
		rrdtool_graph($graph, $final_opts);
	}

}

function graph_halfday($name, $opts)
{
	// Clears file status cache
	clearstatcache();

	// halfday
	$final_opts = ' -w 800 -h 600 -s end-12h -S 120 ' . $opts;
	$graph = getRrdtoolGraphDir() . '/' . $name . '_12h.png';
	// check if the graph already exist, when not create it
	if (file_exists($graph)) {
		// only update graph when old version is older then 10 seconds
		if (time()-filemtime($graph) > 10) {
			rrdtool_graph($graph, $final_opts);
		}
	} else {
		rrdtool_graph($graph, $final_opts);
	}
}

function graph_one_week($name, $opts)
{
	// Clears file status cache
	clearstatcache();

	// one week
	$final_opts = ' -w 800 -h 600 -s end-1w -S 1500 ' . $opts;
	$graph = getRrdtoolGraphDir() . '/' . $name . '_1w.png';
	// check if the graph already exist, when not create it
	if (file_exists($graph)) {
		// only update graph when old version is older then 10 minutes
		if (time()-filemtime($graph) > 10*60) {
			rrdtool_graph($graph, $final_opts);
		}
	} else {
		rrdtool_graph($graph, $final_opts);
	}
}

function graph_one_month($name, $opts)
{
	// Clears file status cache
	clearstatcache();

	$final_opts = ' -w 800 -h 600 -s end-1m -S 7200 ' . $opts;
	$graph = getRrdtoolGraphDir() . '/' . $name . '_5w.png';
	if (file_exists($graph)) {
		// only update graph when old version is older then 10 hours
		if (time()-filemtime($graph) > 10*60*60) {
			rrdtool_graph($graph, $final_opts);
		}
	} else {
		rrdtool_graph($graph, $final_opts);
	}
}

function graph_one_year($name, $opts)
{
	// Clears file status cache
	clearstatcache();

	$final_opts = ' -w 800 -h 600 -s end-400d ' . $opts;
	$graph = getRrdtoolGraphDir() . '/' . $name . '_1y.png';
	if (file_exists($graph)) {
		// only update graph when old version is older then 24 hours
		if (time()-filemtime($graph) > 24*60*60) {
			rrdtool_graph($graph, $final_opts);
		}
	} else {
		rrdtool_graph($graph, $final_opts);
	}
}

function graph_dynamic($name, $opts, $start, $end)
{
	// Clears file status cache
	clearstatcache();

	$final_opts = ' -w 800 -h 600 -s ' . $start . ' -e ' . $end . ' ' .
			$opts;
	$graph = getRrdtoolGraphDir() . '/' . $name . '_dynamic.png';
	rrdtool_graph($graph, $final_opts);
}

?>
