<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * For configuration options we need the string value of the object,
 * so we need to do type casting. Otherwise we cannot store the values in the
 * php session.
 */

// this function tries to find out the basedir of dastralyse and then returns
// the path
function getBaseDir()
{
	$ret;
	$script_dirname = realpath(dirname($_SERVER['SCRIPT_FILENAME']));

	if (preg_match('#htdocs$#', $script_dirname)) {
		$ret = preg_replace('#/[^/]+$#', '', $script_dirname);
	} else if (preg_match('#script$#', $script_dirname)) {
		$ret = preg_replace('#/[^/]+$#', '', $script_dirname);
	} else if (preg_match('#htdocs/js$#', $script_dirname)) {
		$ret = preg_replace('#/[^/]+/[^/]+$#', '', $script_dirname);
	}

	debug_msg(2, 'script_dirname = ' . $script_dirname);
	if (isset($ret) && $ret) {
		debug_msg(2, 'basedir = ' . $ret);
	}

	return $ret;
}

function getCfgfile()
{
	global $cfgfile;

	return $cfgfile;
}

function getDatabaseHost()
{
	global $cfg;
	$host = 'localhost';

	if (isset($cfg->db->dbhost) && $cfg->db->dbhost) {
		$host = (string)$cfg->db->dbhost;
	}

	return $host;
}

function getDatabaseName()
{
	global $cfg;
	$ret = 'pmacct';

	if (isset($cfg->db->dbname) && $cfg->db->dbname) {
		$ret = (string)$cfg->db->dbname;
	}
	debug_msg(2, 'Databasename: ' . $ret);

	return $ret;
}

function getDatabasePassword()
{
	global $cfg;
	$ret = '';

	if (isset($cfg->db->password) && $cfg->db->password) {
		$ret = (string)$cfg->db->password;
	}
	debug_msg(64, 'Databasepass: ' . $ret);

	return $ret;
}

function getDatabasePort()
{
	global $cfg;

	if (getDatabaseType() == 'mysql') {
		$port = '3306';
	}

	if (getDatabaseType() == 'pgsql') {
		$port = '5432';
	}

	if (isset($cfg->db->dbport) && $cfg->db->dbport) {
		$port = (string)$cfg->db->dbport;
	}

	return $port;
}

function getDatabaseType()
{
	global $cfg;
	$type = 'pgsql';

	if (isset($cfg->db->dbtype) && $cfg->db->dbtype) {
		$type = (string)$cfg->db->dbtype;
	}

	return $type;
}

function getDatabaseUser()
{
	global $cfg;
	$ret = 'pmacct';

	if (isset($cfg->db->username) && $cfg->db->username) {
		$ret = (string)$cfg->db->username;
	}
	debug_msg(4, 'Databaseuser: ' . $ret);

	return $ret;
}

function getDebug()
{
	global $DEBUG;

	$level = 0;

	// First check if we have array, because an array has a special
	// meaning
	if (is_array($DEBUG)) {
		// When $DEBUG is an array it is possible to define special
		// debug level for a function. So we need the calling function.
		// If no debug level is defined for the function the generic
		// level will be used

		// We need backtrace to get the calling function name
		$backtrace = debug_backtrace();
		// First we have to check if the calling function is not
		// check_debuglevel or debug_msg, because we need the calling
		// function of check_debuglevel and debug_msg to get the
		// right function name.
		if (isset($backtrace[1]['function']) &&
		    $backtrace[1]['function'] != 'check_debuglevel' &&
		    $backtrace[1]['function'] != 'debug_msg') {
			$function_name = $backtrace[1]['function'];
		} else if (isset($backtrace[2]['function']) &&
			   $backtrace[2]['function'] != 'check_debuglevel' &&
			   $backtrace[2]['function'] != 'debug_msg') {
			$function_name = $backtrace[2]['function'];
		} else if (isset($backtrace[3]['function']) &&
			   $backtrace[3]['function'] != 'check_debuglevel' &&
			   $backtrace[3]['function'] != 'debug_msg') {
			$function_name = $backtrace[3]['function'];
		} else {
			$function_name = 'default';
		}

		// Check if an array element exists with the calling function
		// name. Otherwise use the default debug level.
		if (isset($DEBUG[$function_name])) {
			$level = $DEBUG[$function_name];
		} else if (isset($DEBUG['default'])) {
			$level = $DEBUG['default'];
		} else {
			throw new Exception('Problem with debug variable.');
		}
	} else {
		$level = $DEBUG;
	}

	return $level;
}

function getEthers()
{
	global $cfg;
	$ethers = array();

	if (isset($cfg->network->ethers)) {
		foreach ($cfg->network->ethers as $ether) {
			$ethers[] = (array)$ether;
		}
	}

	return $ethers;
}

function getEtherRXSpeed($name)
{
	$ret = -1;
	$ethers = getEthers();

	foreach ($ethers as $ether) {
		if ($ether['name'] == $name) {
			$ret = $ether['rxspeed'];
		}
	}
	return $ret;
}

function getEtherTXSpeed($name)
{
	$ret = -1;
	$ethers = getEthers();

	foreach ($ethers as $ether) {
		if ($ether['name'] == $name) {
			$ret = $ether['txspeed'];
		}
	}
	return $ret;
}

function getFilter()
{
	$filter = '';
	if (isset($_POST['filter']) && $_POST['filter']) {
		$filter = _stripslashes($_POST['filter']);
	} else if (isset($_SESSION['filter']) && $_SESSION['filter']) {
		$filter = $_SESSION['filter'];
	} else {
		// TODO
		//$filter = 'SELECT * FROM ' . $_SESSION['tablename'] . ' WHERE';
		$filter = 'SELECT * FROM ' . $_SESSION['tablename'] . ' ';
	}

	return $filter;
}

function getFilterFrom()
{
	$from = '';
	if (isset($_SESSION['filterfrom']) && $_SESSION['filterfrom']) {
		$from = $_SESSION['filterfrom'];
	} else {
		if (isset($_SESSION['tablename']) &&
				preg_match('/^acct_v5_(\d+)$/',
					   $_SESSION['tablename'])) {
			$db_time = preg_replace('/^acct_v5_(\d+)$/', '$1',
						 $_SESSION['tablename']);
			$from = date('Y-m-d H:i:s', strtotime($db_time));
		} else {
			$from = date('Y-m-d H:i:s', strtotime('-2 hours'));
		}
	}

	return $from;
}

function getFilterTo()
{
	$to = '';
	if (isset($_SESSION['filterto']) && $_SESSION['filterto']) {
		$to = $_SESSION['filterto'];
	} else {
		if (isset($_SESSION['tablename']) &&
				preg_match('/^acct_v5_(\d+)$/',
					   $_SESSION['tablename'])) {
			$db_time = preg_replace('/^acct_v5_(\d+)$/', '$1',
						 $_SESSION['tablename']);
			// add one day minus 1 second to time
			$to = date('Y-m-d H:i:s', strtotime($db_time)+86399);
		} else {
			$to = date('Y-m-d H:i:s');
		}
	}

	return $to;
}

function getHosts()
{
	global $cfg;
	$hosts = array();

	if (isset($cfg->network->hosts)) {
		foreach ($cfg->network->hosts as $host) {
			$hosts[] = (array)$host;
		}
	}

	return $hosts;
}

function getIncludeFile()
{
	global $include_file;

	return $include_file;
}

function getInterface()
{
	global $cfg;
	$ret = 'eth0';

	if (isset($cfg->network->interface) && $cfg->network->interface) {
		$ret = (string)$cfg->network->interface;
	}

	return $ret;
}

function getNav()
{
	global $topmenu;

	if (isset($_SESSION['nav']) && $_SESSION['nav']) {
		$nav = $_SESSION['nav'];
	} else {
		$nav = $topmenu[0];
	}

	return $nav;
}

function getNetworks()
{
	global $cfg;
	$networks = array();

	if (isset($cfg->network->networks)) {
		foreach ($cfg->network->networks as $network) {
			$networks[] = (array)$network;
		}
	}

	return $networks;
}

function getPmacctdBinary()
{
	global $cfg;

	if (isset($cfg->tools->pmacct)) {
		$ret = (string)$cfg->tools->pmacct;
	} else {
		$ret = '/usr/bin/pmacct';
	}

	return $ret;
}

function getPmacctdCfgfile()
{
	global $pmacctd_cfgfile;

	return $pmacctd_cfgfile;
}

function getSfacctdCfgfile()
{
	global $sfacctd_cfgfile;

	return $sfacctd_cfgfile;
}

function getPmacctdPipeDir()
{
	global $cfg;

	if (isset($cfg->pmacct->pipedir)) {
		$ret = (string)$cfg->pmacct->pipedir;
	} else {
		$ret = '/tmp';
	}

	return $ret;
}

function getPorts()
{
	global $cfg;
	$ports = array();

	if (isset($cfg->network->ports)) {
		foreach ($cfg->network->ports as $port) {
			$ports[] = (array)$port;
		}
	}

	return $ports;
}

// return relative url from path on the server based on running php script
function getRelativeUrl($path)
{
	debug_msg(2, '$path = ' . $path);
	debug_msg(2, 'DOCUMENT_ROOT = ' . $_SERVER['DOCUMENT_ROOT']);

	// directory of the running php script on the server without document
	// root path
	$script_dirname = preg_replace('#' . $_SERVER['DOCUMENT_ROOT'] . '#',
			'', realpath(dirname($_SERVER['SCRIPT_FILENAME'])));

	// directory of $path without document root path
	$url = preg_replace('#' . $_SERVER['DOCUMENT_ROOT'] . '#', '', $path);

	// split paths
	$script_dir_split = preg_split('/\//', $script_dirname);
	$path_split = preg_split('/\//', $url);

	// check if which array has more element and save lower value to $count
	if (count($script_dir_split) > count($path_split)) {
		$count = count($path_split);
	} else if (count($script_dir_split) < count($path_split)) {
		$count = count($script_dir_split);
	} else {
		$count = 0;
	}

	// save position
	for ($i=0; $i<$count; $i++) {
		if ($script_dir_split[$i] != $path_split[$i]) {
			$pos = $i;
			break;
		}
	}

	// if we do not have $pos now the path is a subdirectory under
	// $script_dir
	if (!isset($pos)) {
		$pos=$count;
	}

	debug_msg(2, '$pos = ' . $pos);

	// delete everything which is equal
	for ($i=0; $i<$pos; $i++) {
		array_shift($script_dir_split);
		array_shift($path_split);
	}

	// count leftover elements
	$script_dir_count = count($script_dir_split);
	$path_count = count($path_split);

	debug_msg(2, '$script_dir_count = ' . $script_dir_count);
	debug_msg(2, '$path_count = ' . $path_count);

	// add '..' to $url $path_count $script_dir_count times
	$url = str_repeat('../', $script_dir_count) . implode('/', $path_split);

	debug_msg(2, '$url = ' . $url);

	return($url);

}

function getRrdtoolBinary()
{
	global $cfg;

	if (isset($cfg->tools->rrdtool)) {
		$ret = (string)$cfg->tools->rrdtool;
	} else {
		$ret = '/usr/bin/rrdtool';
	}

	return $ret;
}

function getRrdtoolDbDir()
{
	global $cfg;

	if (isset($cfg->rrd->dbdir)) {
		$ret = (string)$cfg->rrd->dbdir;
	} else {
		$ret = getBaseDir() . '/rrddb';
	}

	return $ret;
}

function getRrdtoolGraphDir()
{
	global $cfg;

	if (isset($cfg->rrd->graphdir)) {
		$ret = (string)$cfg->rrd->graphdir;
	} else {
		$ret = getBaseDir() . '/htdocs/rrd';
	}

	return $ret;
}

function getSelf()
{
	$ret = $_SERVER['PHP_SELF'] . '?id=' . session_id();
	return $ret;
}

function getShow()
{
	global $show;
	if (isset($_SESSION['show']) && $_SESSION['show']) {
		$retval = $_SESSION['show'];
	} else {
		if (isset($show) && is_array($show)) {
			$retval = $show;
		} else {
			error_msg('$show is not an array');
			$retval = array();
		}
	}
	return $retval;
}

function getSort()
{
	$sort = '';
	if (isset($_SESSION['sort']) && $_SESSION['sort']) {
		$sort = $_SESSION['sort'];
	}
	return $sort;
}

function getSortReverse()
{
	$sortr = 0;
	if (isset($_SESSION['sortr']) && $_SESSION['sortr']) {
		$sortr = $_SESSION['sortr'];
	}
	return $sortr;
}

function getSubnav()
{
	global $submenu;

	if (isset($_SESSION['snav']) && $_SESSION['snav']) {
		$snav = $_SESSION['snav'];
	} else {
		$snav = $submenu[getNav()][0];
	}

	return $snav;
}

function getTableStart()
{
	$tblstart = 0;
	if (isset($_SESSION['tblstart']) && $_SESSION['tblstart']) {
		if ($_SESSION['tblstart'] < 0) {
			$tblstart = 0;
			setTableStart(0);
		} else {
			$tblstart = $_SESSION['tblstart'];
		}
	}
	return $tblstart;
}

function getTopLimit()
{
	global $cfg;
	$limit = 50;

	if (isset($cfg->toplimit) && $cfg->toplimit) {
		$limit = $cfg->toplimit;
	}

	return $limit;
}

function setCfgfile($file)
{
	global $cfgfile;

	$cfgfile = $file;
}


function setDatabaseHost($host)
{
	global $cfg;

	$cfg->db->dbhost = $host;
}

function setDatabaseName($name)
{
	global $cfg;

	$cfg->db->dbname = $name;
}

function setDatabasePassword($password)
{
	global $cfg;

	$cfg->db->password = $password;
}

function setDatabasePort($port)
{
	global $cfg;

	$cfg->db->dbport = $port;
}

function setDatabaseType($type)
{
	global $cfg;

	$cfg->db->dbtype = $type;
}

function setDatabaseUser($user)
{
	global $cfg;

	$cfg->db->username = $user;
}

// Structure of $ethers
// Array
// (
//    [0] => Array ( [mac] => value1, [key2] => value2, [key3] => value3... )
//    [1] => Array ( [mac] => value1, [key2] => value2, [key3] => value3... )
//     ....
// )
function setEthers($ethers)
{
	global $cfg;

	unset($cfg->network->ethers);
	foreach ($ethers as $key1 => $values) {
		if (isset($values['mac']) && $values['mac']) {
			foreach ($values as $key2 => $value) {
				if (isset($value) && $value) {
					$cfg->network->ethers[$key1]->$key2 =
						$value;
				}
			}
		}
	}

}

function setFilter($filter)
{
	if (isset($_POST['clearfilter']) && $_POST['clearfilter']) {
		unset($_SESSION['filter']);
	}
	$_SESSION['filter'] = $filter;
}

function setFilterFrom($from)
{
	if (isset($_POST['clearfilter']) && $_POST['clearfilter']) {
		unset($_SESSION['filterfrom']);
	} else {
		$_SESSION['filterfrom'] = $from;
	}
}

function setFilterTo($to)
{
	if (isset($_POST['clearfilter']) && $_POST['clearfilter']) {
		unset($_SESSION['filterto']);
	} else {
		$_SESSION['filterto'] = $to;
	}
}

// Structure of $hosts
// Array
// (
//    [0] => Array ( [net] => value1, [key2] => value2, [key3] => value3... )
//    [1] => Array ( [net] => value1, [key2] => value2, [key3] => value3... )
//     ....
// )
function setHosts($hosts)
{
	global $cfg;

	unset($cfg->network->hosts);
	foreach ($hosts as $key1 => $values) {
		if (isset($values['host']) && $values['host']) {
			foreach ($values as $key2 => $value) {
				if (isset($value) && $value) {
					$cfg->network->hosts[$key1]->$key2 =
						$value;
				}
			}
		}
	}
}

function setIncludeFile($file)
{
	global $include_file;

	$include_file = $file;
}

function setInterface($interface)
{
	global $cfg;

	$cfg->network->interface = $interface;
}

function setNav($setnav)
{
	// Commit navigation point to session
	$_SESSION['nav'] = $setnav;
}

// Structure of $nets
// Array
// (
//    [0] => Array ( [net] => value1, [key2] => value2, [key3] => value3... )
//    [1] => Array ( [net] => value1, [key2] => value2, [key3] => value3... )
//     ....
// )
function setNetworks($nets)
{
	global $cfg;

	unset($cfg->network->networks);
	foreach ($nets as $key1 => $values) {
		if (isset($values['net']) && $values['net']) {
			foreach ($values as $key2 => $value) {
				if (isset($value) && $value) {
					$cfg->network->networks[$key1]->$key2 =
						$value;
				}
			}
		}
	}
}

function setPmacctdBinary($pmacct)
{
	global $cfg;

	$cfg->tools->pmacct = $pmacct;
}

function setPmacctdPipeDir($pmacctpipe)
{
	global $cfg;

	$cfg->pmacct->pipedir = $pmacctpipe;
}

// Structure of $ports
// Array
// (
//    [0] => Array ( [port] => value1, [key2] => value2, [key3] => value3... )
//    [1] => Array ( [port] => value1, [key2] => value2, [key3] => value3... )
//     ....
// )
function setPorts($ports)
{
	global $cfg;

	unset($cfg->network->ports);
	foreach ($ports as $key1 => $values) {
		if (isset($values['port']) && $values['port']) {
			foreach ($values as $key2 => $value) {
				if (isset($value) && $value) {
					$cfg->network->ports[$key1]->$key2 =
						$value;
				}
			}
		}
	}
}

function setRrdtoolBinary($rrd)
{
	global $cfg;

	$cfg->tools->rrdtool = $rrd;
}

function setRrdtoolDbDir($rrddb)
{
	global $cfg;

	$cfg->rrd->dbdir = $rrddb;
}

function setRrdtoolGraphDir($rrdgraph)
{
	global $cfg;

	$cfg->rrd->graphdir = $rrdgraph;
}

function setShow($setshow)
{
	if (is_array($setshow)) {
		$_SESSION['show'] = $setshow;
	} else {
		error_msg('setShow called with non array argument');
	}
}

function setSort($setsort)
{
	// Commit sort point to session
	$_SESSION['sort'] = $setsort;
}

function setSortReverse()
{
	if (isset($_SESSION['sortr']) && $_SESSION['sortr'] == 1) {
		$_SESSION['sortr'] = 0;
	} else {
		$_SESSION['sortr'] = 1;
	}
}

function setSubnav($setsnav)
{
	// Commit navigation point to session
	$_SESSION['snav'] = $setsnav;
}

function setTableStart($tablestart)
{
	$_SESSION['tblstart'] = $tablestart;
}

function setTopLimit($limit)
{
	global $cfg;

	$cfg->toplimit = $limit;
}

?>
