<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Open database connection
function db_connect()
{
	$dbtype = getDatabaseType();
	$dbuser = getDatabaseUser();
	$dbpass = getDatabasePassword();
	$dbhost = getDatabaseHost();
	$dbport = getDatabasePort();
	$dbname = getDatabaseName();
	debug_msg(2, '$dbtype = ' . $dbtype);
	debug_msg(2, '$dbhost = ' . $dbhost);
	debug_msg(2, '$dbport = ' . $dbport);
	debug_msg(2, '$dbname = ' . $dbname);
	debug_msg(4, '$dbuser = ' . $dbuser);
	debug_msg(64, '$dbpass = ' . $dbpass);

	if ($dbtype == 'mysql') {
		$db = _mysql_db_connect();
	} else if ($dbtype == 'pgsql') {
		$db = _pg_db_connect();
	} else {
		// Fall back to pear if we have a unsupported database type.
		// This is only here for test purposes.
		$db = _pear_db_connect();
	}

	return($db);

}

function db_disconnect($db)
{
	$dbtype = getDatabaseType();

	if ($dbtype == 'mysql') {
		mysql_close($db);
	} else if ($dbtype == 'pgsql') {
		pg_close($db);
	} else {
		// Fall back to pear if we have a unsupported database type
		$db->disconnect();
	}

}

// Send query to database and return results as array.
// e.g.
// * Array(
// *   Array('agent_id' => 0, ..., 'mac_src' => '02:b3:d5:f0:00:94'),
// *   Array('agent_id' => 0, ..., 'mac_src' => '02:f6:65:35:65:67'),
// *   ....
// * );
function db_query($query)
{
	$results = Array();
	$dbtype = getDatabaseType();

	$db = db_connect();

	debug_msg(2, '$query = ' . $query);

	if ($dbtype == 'mysql') {
		$qresult = _mysql_db_query($query);
		while ($row = mysql_fetch_array($qresult, MYSQL_ASSOC)) {
			$results[] = $row;
		}
	} else if ($dbtype == 'pgsql') {
		$qresult = _pg_db_query($query);
		while ($row = pg_fetch_array($qresult, NULL, PGSQL_ASSOC)) {
			$results[] = $row;
		}
	} else {
		// Fall back to pear if we have a unsupported database type
		$qresult = $db->query($query);
		if (DB::isError($query)) {
			error_msg($query->getMessage());
		}
		while ($row = $qresult->fetchRow(DB_FETCHMODE_ASSOC)) {
			$results[] = $row;
		}
	}

	if (isset($results[0]) && $results[0]) {
		debug_msg(2, 'First result:');
		foreach ($results[0] as $key => $value) {
			debug_msg(2, $key . ' = ' . $value);
		}
	} else {
		debug_msg(2, 'no results');
	}

	db_disconnect($db);

	// Save results in SESSION
	$_SESSION['results'] = $results;

	return($results);
}

// escape strings to prevent sql injection
function db_escape_string($string)
{
	$dbtype = getDatabaseType();
	$db = db_connect();

	if ($dbtype == 'mysql') {
		$escape_string =
			mysql_real_escape_string(_stripslashes($string), $db);
	} else if ($dbtype == 'pgsql') {
		$escape_string =
			pg_escape_string($db, _stripslashes($string));
	} else {
		// TODO PEAR
		$escape_string = $string;
	}

	db_disconnect($db);

	return($escape_string);
}

// This functions shows db_query results as a html table.
// The first arguments tells the function where to append
// the domchild object.
function show_db_query_results($level, $results, $perpage)
{
	$i = 0;

	// How many entries should be shown per page
	if (!isset($perpage) || $perpage == 0) {
		$perpage = 10;
	}

	$tblprev = 0;

	if (isset($_GET['tblstart'])) {
		// Save startpoint of table
		setTableStart($_GET['tblstart']);
	}

	// reset TableStart if value is greater then the results we have
	if (count($results) < getTableStart()+$perpage) {
		setTableStart(0);
	}

	if (getTableStart() >= $perpage) {
		$tblprev = getTableStart() - $perpage;
	}

	debug_msg(3, 'tblstart = ' . getTableStart());
	debug_msg(3, 'tblprev = ' . $tblprev);

	domchild($level, 'div', array('class' => 'sqlnav'), '');

	if (getTableStart() > 0) {
		domchild($level+1, 'a', array('href' => getSelf() .
					      '&tblstart=' .
					      rawurlencode($tblprev)), '');
		domchild($level+2, 'img', array('src' => 'png/uarr.png',
						'alt' => 'Previous'), '');
	} else {
		domchild($level+1, 'img', array('src' => 'png/uarrl.png',
					 'alt' => 'Previous'), '');
	}

	domchild($level+1, 'span', '', '|');

	if (count($results) > getTableStart() + $perpage) {
		domchild($level+1, 'a', array('href' => getSelf() .
					      '&tblstart=' .
					      rawurlencode(getTableStart()+$perpage)), '');
		domchild($level+2, 'img', array('src' => 'png/darr.png',
						'alt' => 'Next'), '');
	} else {
		domchild($level+1, 'img', array('src' => 'png/darrl.png',
					 'alt' => 'Next'), '');
	}

	if (isset($results[0]) && $results[0]) {
		domchild($level, 'form', array('action' => getSelf(),
					       'method' => 'post'), '');
		domchild($level+1, 'table', '', '');
		domchild($level+2, 'tr', '', '');
		domchild($level+3, 'td', array('class' => 'top'), '');
		domchild($level+4, 'h3', '', 'Show:');
		foreach ($results[0] as $key => $value) {
			$i++;
			// Check if $i is odd or even
			if ($i&1) {
				// odd
				domchild($level+3, 'td', '', '');
			} else {
				// even
				domchild($level+4, 'br', '', '');
			}
			if (isset($_POST['show']) && is_array($_POST['show'])) {
				setShow($_POST['show']);
			}
			$checked = 0;
			foreach (getShow() as $show) {
				if ($show == $key) {
					$checked = 1;
				}
			}
			if ($checked == 1) {
				domchild($level+4, 'input',
					 array('type' => 'checkbox',
					       'name' => 'show[]',
					       'value' => $key,
					       'onclick' => 'this.form.submit();',
					       'onchange' => 'this.form.submit();',
					       'checked' => 'checked'), '');
			} else {
				domchild($level+4, 'input',
					 array('type' => 'checkbox',
					       'name' => 'show[]',
					       'value' => $key,
					       'onclick' => 'this.form.submit();',
					       'onchange' => 'this.form.submit()'),
					 '');
			}
			domchild($level+4, 'span', '', $key);
		}
		domchild($level+3, 'td', array('class' => 'padleft'), '');
		domchild($level+4, 'noscript', '', '');
		domchild($level+5, 'div', '', '');
		domchild($level+6, 'input', array('type' => 'submit', 
						  'value' => 'Change'), '');

		domchild($level, 'table', array('class' => 'table'), '');
		domchild($level+1, 'tr', '', '');
		domchild($level+2, 'th', array('class' => 'nowrap'), '');
		domchild($level+3, 'a',
			 array('href' => getSelf() .
			       '&sort=' . rawurlencode('id')), 'id');
		if (getSort() == '' || getSort() == 'id') {
			// id has only one order so we have to reset the order
			// to 0
			if (getSortReverse() == 1) {
				setSortReverse();
			}
			domchild($level+3, 'img',
				 array('src' => 'png/uarr-small.png',
				 'alt' => ' down '), '');
		}

		foreach ($results[0] as $key => $value) {
			$checked = 0;
			foreach (getShow() as $show) {
				if ($show == $key) {
					$checked = 1;
				}
			}
			if ($checked == 1) {
				domchild($level+2, 'th',
					 array('class' => 'nowrap'), '');
				domchild($level+3, 'a',
					 array('href' => getSelf() .
					       '&sort=' .
					       rawurlencode((string)$key)),
					 (string)$key);
				if (getSort() == (string)$key) {
					if (getSortReverse() == 1) {
						$IMG = 'png/darr-small.png';
					} else {
						$IMG = 'png/uarr-small.png';
					}
					domchild($level+3, 'img',
						 array('src' => $IMG,
						       'alt' => ' up '), '');
				}
			}
		}
		domchild($level+2, 'th',
			 array('class' => 'nowrap'), '');

		//for ($i = 0; $i < count($results); $i++) {
		for ($i = getTableStart(); $i < getTableStart()+$perpage; $i++) {
			// Check if $i is odd or even
			if ($i&1) {
				// odd
				domchild($level+1, 'tr', '', '');
			} else {
				// even
				domchild($level+1, 'tr',
					array('class' => 'even'), '');
			}
			if (isset($_GET['start'])) {
				$start = intval($_GET['start']);
			} else {
				$start = 0;
			}
			domchild($level+2, 'td', '', $i + 1 + $start);
			foreach ($results[$i] as $key => $value) {
				$unit = ' ';
				// TODO remove after project presentation
				// anonymise ip addresses
				//if (preg_match('/ip_(src|dst)/', $key)) {
				//	$value=preg_replace('/\.\d+$/', '.xxx',
				//			    $value);
				//}
				// Check if value is a number or a numeric
				// string and convert it to kb, mb, gb
				if (is_numeric($value)) {
					if ($value > 99999) {
						$value = intval($value / 1000);
						$unit = ' K';
					}
					if ($value > 99999) {
						$value = intval($value / 1000);
						$unit = ' M';
					}
					if ($value > 99999) {
						$value = intval($value / 1000);
						$unit = ' G';
					}
					$value .= $unit;
				}
				if (is_ip($value)) {
					$dnsname = gethostbyaddr($value);
					if (isset($dnsname)) {
						$value .= ' (' . $dnsname . ')';
					}
				}
				$checked = 0;
				foreach (getShow() as $show) {
					if ($show == $key) {
						$checked = 1;
					}
				}
				if ($checked == 1) {
					domchild($level+2, 'td', '', $value);
				}
			}
			// TODO Details
			/*
			domchild($level+2, 'td', array('class' => 'nowrap'), 
				 '');
			domchild($level+3, 'form', array('action' => getSelf(),
						         'method' => 'post'),
				 '');
			domchild($level+4, 'div', '', '');
			domchild($level+5, 'input', array('type' => 'submit', 
							  'value' => 'Details'),
				 '');
			*/
		}
	} else {
		domchild($level, 'h1', array('class' => 'center'),
			'No results');
	}

	domchild($level, 'div', array('class' => 'sqlnav'), '');

	if (getTableStart() > 0) {
		domchild($level+1, 'a', array('href' => getSelf() .
					      '&tblstart=' .
					      rawurlencode($tblprev)), '');
		domchild($level+2, 'img', array('src' => 'png/uarr.png',
						'alt' => 'Previous'), '');
	} else {
		domchild($level+1, 'img', array('src' => 'png/uarrl.png',
					 'alt' => 'Previous'), '');
	}

	domchild($level+1, 'span', '', '|');

	if (count($results) > getTableStart() + $perpage) {
		domchild($level+1, 'a', array('href' => getSelf() .
					      '&tblstart=' .
					      rawurlencode(getTableStart()+$perpage)), '');
		domchild($level+2, 'img', array('src' => 'png/darr.png',
						'alt' => 'Next'), '');
	} else {
		domchild($level+1, 'img', array('src' => 'png/darrl.png',
					 'alt' => 'Next'), '');
	}

}

function _mysql_db_connect()
{
	$db = mysql_connect(getDatabaseHost() . ':' . getDatabasePort(),
		getDatabaseUser(), getDatabasePassword());

	if (!$db) {
		throw new Exception('MySQL connection database error');
	}

	if (!mysql_select_db(getDatabaseName())) {
		throw new Exception('MySQL connection database error: ' . 
			mysql_error());
	}
	
	return($db);

}

function _pg_db_connect()
{
	$conn_string = 'host=' . getDatabaseHost() .
		' port=' .  getDatabasePort() .
		' dbname=' . getDatabaseName() .
		' user=' . getDatabaseUser() .
		' password=' . getDatabasePassword();

	debug_msg(64, 'conn_string = ' . $conn_string);

	$db = pg_connect($conn_string);

	if (!$db) {
		throw new Exception('Postgres connection database error'); 
	}

	return($db);

}

function _pear_db_connect()
{
	$dbtype = getDatabaseType();
	$dbuser = getDatabaseUser();
	$dbpass = getDatabasePassword();
	$dbhost = getDatabaseHost();
	$dbport = getDatabasePort();
	$dbname = getDatabaseName();

	$dbdsn = $dbtype . '://' . $dbuser . ':' . $dbpass . '@' .
		$dbhost . ':' . $dbport . '/' . $dbname;

	debug_msg(64, 'dbdsn = ' . $dbdsn);

	#$db =& DB::connect($dbdsn);
	#if (PEAR::isError($db)) {
	#	error_msg($db->getMessage());
	#}
	return($db);
}

function _mysql_db_query($query)
{
	$result = mysql_query($query);
	if (!$result) {
		throw new Exception(mysql_error());
	}

	return($result);
}

function _pg_db_query($query)
{
	$result = pg_query($query);
	if (!$result) {
		throw new Exception(pg_last_error());
	}

	return($result);
}

// TODO
// Convert filter to sql where clause
// Relation tags: '==', '!=', '>', '<', '>=', '<='
function filter2sql()
{
	$proto = array();
	$sub = array();
	$rel = array();
	$value = array();
	$count = 0;
	$filter = getFilter();

	#$pattern = '/^\s*([a-z]+)\.([a-z]+)\s*(==|!=|>|<|>=|<=)\s*([^&|]+)/i';
	$pattern = '/^\s*([a-z]+)\.([a-z]+)\s*(==|!=|>|<|>=|<=)\s*([^\s]+)/i';

	$array = split('(&&|\|\|)', $filter);

	//while ($filter) {
	foreach ($array as $filter) {
		debug_msg(2, '$filter = ' . $filter);

		if (preg_match($pattern, $filter, $match)) {
			if ($match[1]) {
				debug_msg(2, '$match[1] = ' . $match[1]);
				$proto[] = $match[1];
				$filter = preg_replace('/^' . $match[1] . '\./i', '', $filter);
			}
			if ($match[2]) {
				debug_msg(2, '$match[2] = ' . $match[2]);
				$sub[] = $match[2];
				$filter = preg_replace('/^' . $match[2] . '\s*/i', '', $filter);
			}
			if ($match[3]) {
				debug_msg(2, '$match[3] = ' . $match[3]);
				$rel[] = $match[3];
				$filter = preg_replace('/^' . $match[3] . '\s*/i', '', $filter);
			}
			if ($match[4]) {
				debug_msg(2, '$match[4] = ' . $match[4]);
				$value[] = $match[4];
				$filter = preg_replace('/^' . $match[4] . '\s*/i', '', $filter);
			}
		}
		if ($count > 10) {
			return(false);
		}
		$count++;
	}
	// TODO Added 'AND ' to $filter
	return(' AND ' . $filter);
}

// TODO
// This function returns all tables in a database as a array
function get_sql_tables()
{
	$ret;
	$dbtype = getDatabaseType();

	if ($dbtype == 'mysql') {
		$ret = db_query('show tables');
	} else if ($dbtype == 'pgsql') {
		// magic select statement from pear
		$query='SELECT c.relname AS "Name"'
			. ' FROM pg_class c, pg_user u'
			. ' WHERE c.relowner = u.usesysid'
			. " AND c.relkind = 'r'"
			. ' AND NOT EXISTS'
			. ' (SELECT 1 FROM pg_views'
			. '  WHERE viewname = c.relname)'
			. " AND c.relname !~ '^(pg_|sql_)'"
			. ' UNION'
			. ' SELECT c.relname AS "Name"'
			. ' FROM pg_class c'
			. " WHERE c.relkind = 'r'"
			. ' AND NOT EXISTS'
			. ' (SELECT 1 FROM pg_views'
			. '  WHERE viewname = c.relname)'
			. ' AND NOT EXISTS'
			. ' (SELECT 1 FROM pg_user'
			. '  WHERE usesysid = c.relowner)'
			. " AND c.relname !~ '^pg_'";
		$ret = db_query($query);
	} else {
		// Fall back to pear if we have a unsupported database type
		$db = db_connect;
		$ret = $db->getListOf('tables');
		db_disconnect($db);
	}

	return $ret;
}

// This function returns all tables in a database like $1
function get_sql_tables_like($like)
{
	$ret;
	$dbtype = getDatabaseType();

	if ($dbtype == 'mysql') {
		$ret = db_query("show tables like '" . $like . "'");
	} else if ($dbtype == 'pgsql') {
		$query  = 'SELECT tablename from pg_tables where ';
		$query .= "schemaname='public' and ";
		$query .= "tablename LIKE '" . $like . "' ";
		$query .= "order by tablename DESC";
		$ret = db_query($query);
	} else {
		// Fall back to pear if we have a unsupported database type
		//$db = db_connect;
		//$ret = $db->getListOf('tables');
		//db_disconnect($db);
		// TODO
	}

	return $ret;
}


?>
