<?php
/*
 * Copyright (C) 2009 Patrick Ditzel
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

function html_dom()
{
	global $dom;

	// create an instance of the DOMImplementation class
	$imp = new DOMImplementation();

	// create a DOMDocumentType instance
	$dtd = $imp->createDocumentType('html',
		'-//W3C//DTD XHTML 1.0 Strict//EN',
		'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd');

	// create a DOMDocument instance
	$dom[0] = $imp->createDocument('', '', $dtd);

	// set other properties
	$dom[0]->encoding = 'UTF-8';
	$dom[0]->version = '1.0';
	$dom[0]->formatOutput = true;
	$dom[0]->preserveWhiteSpace = false;

	// create root element
	domchild(0, 'html', array('lang' => 'en'), '');

}

function domchild($level, $element, $attribute, $text)
{
	global $dom;

	// Create new child element
	$domchild = $dom[0]->createElement($element);
	// Append child to parent
	$dom[$level]->appendChild($domchild);

	if (check_debuglevel(32)) {
		$comment = 'domchild:start';
		$domdebugchild = $dom[0]->createComment($comment);
		$domchild->appendChild($domdebugchild);
		$comment = 'level = ' . $level;
		$domdebugchild = $dom[0]->createComment($comment);
		$domchild->appendChild($domdebugchild);
		$domdebugchild = $dom[0]->createComment('text = \''  .
			$text . '\'');
		$domchild->appendChild($domdebugchild);
		$comment = 'domchild:end';
		$domdebugchild = $dom[0]->createComment($comment);
		$domchild->appendChild($domdebugchild);
	}

	if (check_debuglevel(128)) {
		print "<pre id=debug>\n";
		print "domchild: level = $level\n";
		print "domchild: text = $text\n";
		print "</pre>";
	}

	// Check if attributes is set, is an array and has content
	if (isset($attribute) && is_array($attribute) && $attribute) {
		foreach (array_keys($attribute) as $param) {

			// Create new attribute element
			$domchild_attr = $dom[0]->createAttribute($param);
			// Append attribute to above created child
			$domchild->appendChild($domchild_attr);

			// Create value for attribute element
			$domchild_val =
				 $dom[0]->createTextNode($attribute[$param]);
			// Append value to attribute
			$domchild_attr->appendChild($domchild_val);

		}
	}

	// Check if text is set and has content
	if (isset($text) && $text) {
		$domchild_text = $dom[0]->createTextNode($text);
		$domchild->appendChild($domchild_text);
	}

	// !!! IMPORTANT !!!
	// Add created child element to dom array
	// keep childrefence in global namespace for the created domchildobject
	// the level points to the position where dependent childs has to append
	$dom[$level+1] = $domchild;

	// Clean up old domchilds
	cleanup_dom($level+2, $level);

}

function cleanup_dom($start_level, $level)
{
	global $dom;

	if (check_debuglevel(32)) {
		$comment = 'cleanup_dom:start';
		$domdebugchild = $dom[0]->createComment($comment);
		$dom[$level+1]->appendChild($domdebugchild);
		$comment = 'level = ' . $level;
		$domdebugchild = $dom[0]->createComment($comment);
		$dom[$level+1]->appendChild($domdebugchild);
		$comment = 'start_level = ' . $start_level;
		$domdebugchild = $dom[0]->createComment($comment);
		$dom[$level+1]->appendChild($domdebugchild);
		$comment = 'dom_count (before) = ' . count($dom);
		$domdebugchild = $dom[0]->createComment($comment);
		$dom[$level+1]->appendChild($domdebugchild);
	}
	if (check_debuglevel(128)) {
		echo "<pre id='debug'>";
		echo 'level = ' . $level . "\n";
		echo 'start_level = ' . $start_level . "\n";
		echo 'dom_count (before) = ' . count($dom) . "\n";
	}
	if (count($dom) > $start_level) {
		for ($i = count($dom)-1; $i >= $start_level; $i--) {
			if (check_debuglevel(32)) {
				$comment = 'unset(dom[' . $i . ']';
				$domdebugchild = $dom[0]->createComment($comment);
				$dom[$level+1]->appendChild($domdebugchild);
			}
			if (check_debuglevel(128)) {
				echo "unset(dom)" . $i . "\n";
			}
			unset($dom[$i]);
		}
	}
	if (check_debuglevel(128)) {
		echo 'dom_count (after) = ' . count($dom) . "\n";
		echo "</pre>";
	}
	if (check_debuglevel(32)) {
		$comment = 'dom_count (after) = ' . count($dom);
		$domdebugchild = $dom[0]->createComment($comment);
		$dom[$level+1]->appendChild($domdebugchild);
		$comment = 'cleanup_dom:end';
		$domdebugchild = $dom[0]->createComment($comment);
		$dom[$level+1]->appendChild($domdebugchild);
	}
}

function import_dom_node($level, $file)
{
	global $dom;

	if (file_exists($file)) {
	
		$htmlfile = new DOMDocument();
		$htmlfile->formatOutput = true;
		$htmlfile->preserveWhiteSpace = false;
		if ($htmlfile->load($file)) {

			// We want to import the node at the first appearance
			// of 'div'
			$node = $htmlfile->getElementsByTagName('div')->item(0);

			// Import the node, and all its children, to the document
			$node = $dom[0]->importNode($node, true);
			// And then append it to the main node
			$dom[$level]->appendChild($node);

		}
			
	} else {

		 error_msg('No such file: ' . $file);

	}

}

function show_html_page($dom)
{
	$set_xhtml_header = 0;

	// Check if browser accepts xhtml+xml
	if (preg_match('#application/xhtml\+xml#i', $_SERVER['HTTP_ACCEPT'])) {
		// Only set xhtml header when debug level is not set to 128
		if (check_debuglevel(128)) {
				$set_xhtml_header = 0;
		} else {
				$set_xhtml_header = 1;
		}
	}

	// When debug mode is higher or equal then 16 verify the xhtml object
	if (check_debuglevel(16)) {
		$xsd_url='http://www.w3.org/2002/08/xhtml/xhtml1-strict.xsd';
		// We need to save the current dom object as a temporary
		// file and then load it again. This seems to be a bug.
		$temp = tempnam('/tmp', 'xhtml-');
		$dom->save($temp);
		$tmpdom = new DOMDocument();
		$tmpdom->load($temp);
		unlink($temp);
		if ($tmpdom->schemaValidate($xsd_url)) {
			debug_msg(16, 'generated xhtml code is valid');
		} else {
			error_msg('generated xhtml code is not valid');
			//$set_xhtml_header = 0;
		}
	}

	if ($set_xhtml_header == 1) {
		// Check if headers are already send
		if (headers_sent()) {
			error_msg('Cannot set content-type to: ' .
					'application/xhtml+xml');
		} else {
			header('Content-Type: application/xhtml+xml');
		}
	}

	// Error messages have to be added to the dom object as late as
	// possible to catch all error messages. Every error which will
	// occur after create_error_msg will be silently ignored.
	create_error_msg();

	echo $dom->saveXML();

	// Flush the output buffer
	flush();
}

?>
