// SVG namespace
var svgNS = "http://www.w3.org/2000/svg";
// the SVG document handler
var documentSVG = null;
// will store the root <g> element that groups all chart elements
var chartGroup = null;
// how often to request new data from server?
var updateInterval = 1000;
// coordinates (in pixels) used to translate the chart
var x = 10, y = 10;
// chart's dimension (in pixels)
var height = 42, width = 500;
// chart's axis origin
var xt1 = 0;
// chart's axis maximum values
var xt2 = 100;
// number of horizontal and vertical axis divisions
var xDivisions = 10, yDivisions = 1;
// default text width and height for initial display (recalculated
// afterwards)
var defaultTextWidth = 30, defaultTextHeight = 20;
// will retain references to the chart units for recalculating positions
var xIndexes = new Array(xDivisions + 1);
// will store the text node that displays the selected chart node
var currentNodeInfo;
// shared svg elements
var chartGroup, dataGroup, dataPath;
// Percent text
var text1;
var text2;
// Percent rect
var rect1;
var rect2;
var rect3;

// initializes chart
function init(evt)
{
	/**** Prepare the group that will contain all chart data  ****/
	// obtain SVG document handler
	documentSVG = evt.target.ownerDocument;
	// create the <g> element that groups all chart elements
	chartGroup = documentSVG.createElementNS(svgNS, "g");
	chartGroup.setAttribute("transform", "translate(" + x + " " + y + ")");

	/*** Prepare the group that will store the Y and Y axis and numbers ***/
	axisGroup = documentSVG.createElementNS(svgNS, "g");
	// create the X axis line as a <path> element
	axisPath = documentSVG.createElementNS(svgNS, "path");
	// the axis lines will be black, 2 pixels wide
	axisPath.setAttribute("stroke", "black");
	axisPath.setAttribute("stroke-width", "2");

	/**** Create the division lines for the X and Y axis  ****/
	// create the path definition text for the X axis division lines
	pathText = "M 0 " + height;
	// adds divisions to the X axis (differently for last division)
	for (var i = 0; i <= xDivisions; i++)
		pathText += "l 0 5 l 0 -5 " +
			((i == xDivisions) ? "" : ("l " + width/xDivisions +
						   " 0"));
	// create the path definition text for the Y axis division lines
	pathText += "M 0 " + height;
	// adds one division to the Y axis (differently for last division)
	pathText += "l 0 0 l 0 0 " + ("l 0 -" + height);
	// add the path definition (the <d> attribute) to the path
	axisPath.setAttribute("d", pathText);
	// add the path to the axis group
	axisGroup.appendChild(axisPath);

	/**** Create the text nodes for the X and Y axis  ****/
	// adds text nodes for the X axis
	for (var i = 0; i <= xDivisions; i++) {
		// creates the <text> node for the division
		t = documentSVG.createElementNS(svgNS, "text");
		// stores the node for future reference
		xIndexes[i] = t;
		// creates the text for the <text> node
		t.appendChild(documentSVG.createTextNode(
							 (xt1 + i *
							  ((xt2 - xt1) /
							  xDivisions) + '%')));
		// sets the X and Y attributes for the <text> node
		t.setAttribute("x", i * width / xDivisions -
			       defaultTextWidth / 2);
		t.setAttribute("y", height + 30 + defaultTextHeight);
		// when the graph first loads, we want the text nodes invisible
		t.setAttribute("stroke", "white");
		// add the <text> node to the axis group
		axisGroup.appendChild(t);
	}

	// add the axis group to the chart
	chartGroup.appendChild(axisGroup);

	/**** Prepare the <path> element that will draw chart's data ****/
	dataPath = documentSVG.createElementNS(svgNS, "path");
	dataPath.setAttribute("stroke", "black");
	dataPath.setAttribute("stroke-width", "1");
	dataPath.setAttribute("fill", "none");
	// add the data path to the chart group
	chartGroup.appendChild(dataPath);

	/**** Final initialization steps ****/
	// add the chart group to the SVG document
	documentSVG.documentElement.appendChild(chartGroup);
	// this is needed to correctly display text nodes in Firefox
	setTimeout("refreshXYIndexes()", 500);
	// update rrd databases
	do_rrdtool_update_port_dbs();
	// initiate repetitive server requests
	setTimeout('do_interface_counter();', updateInterval);
}

// this function redraws the text for the axis units and makes it visible
// (this is required to correctly position the text in Firefox)
function refreshXYIndexes()
{
	// redraw text nodes on the X axis
	for (var i = 0; i <= xDivisions; i++)
		if (typeof xIndexes[i].getBBox != "undefined")
			try
			{
				textWidth = xIndexes[i].getBBox().width;
				textHeight = xIndexes[i].getBBox().height;
				xIndexes[i].setAttribute("x", i *
							 width/xDivisions -
							 textWidth/2);
				xIndexes[i].setAttribute("y", height + 10 +
							 textHeight);
				xIndexes[i].setAttribute("stroke", "black");
			}
			catch(e) {}
}

function do_interface_counter_cb(countera) {

	var dev = countera['dev'];
	var percent = countera['percent'];
	var txpercent = countera['txpercent'];
	var rxpercent = countera['rxpercent'];
	var rate = countera['rate'];
	var txrate = countera['txrate'];
	var rxrate = countera['rxrate'];
	var unit = 'bit/s';
	var txunit = 'bit/s';
	var rxunit = 'bit/s';

	// remove old content
	if (text1) {
		chartGroup.removeChild(text1);
		text = null;
	}
	if (text2) {
		chartGroup.removeChild(text2);
		text2 = null;
	}
	if (rect1) {
		chartGroup.removeChild(rect1);
		rect1 = null;
	}
	if (rect2) {
		chartGroup.removeChild(rect2);
		rect2 = null;
	}
	if (rect3) {
		chartGroup.removeChild(rect3);
		rect3 = null;
	}

	/* Disable total graph for now
	rect1 = documentSVG.createElementNS(svgNS, "rect");
	rect1.setAttributeNS(null, 'x', 1);
	rect1.setAttributeNS(null, 'y', 1);
	rect1.setAttributeNS(null, 'height', 40);
	rect1.setAttributeNS(null, 'width', percent * 5);
	rect1.setAttributeNS(null, 'fill', 'blue');
	chartGroup.appendChild(rect1);
	*/

	if (countera['maxrxrate'] > 0) {
		rect2 = documentSVG.createElementNS(svgNS, "rect");
		rect2.setAttributeNS(null, 'x', 1);
		rect2.setAttributeNS(null, 'y', 1);
		if (countera['maxtxrate'] > 0) {
			rect2.setAttributeNS(null, 'height', 20);
		} else {
			rect2.setAttributeNS(null, 'height', 40);
		}
		rect2.setAttributeNS(null, 'width', rxpercent * 5);
		rect2.setAttributeNS(null, 'fill', 'palegreen');
		rect2.setAttributeNS(null, 'fill-opacity', '0.6');
		chartGroup.appendChild(rect2);
	}

	if (countera['maxtxrate'] > 0) {
		rect3 = documentSVG.createElementNS(svgNS, "rect");
		rect3.setAttributeNS(null, 'x', 1);
		if (countera['maxrxrate'] > 0) {
			rect3.setAttributeNS(null, 'y', 21);
		} else {
			rect3.setAttributeNS(null, 'y', 1);
		}
		rect3.setAttributeNS(null, 'width', txpercent * 5);
		if (countera['maxrxrate'] > 0) {
			rect3.setAttributeNS(null, 'height', 20);
		} else {
			rect3.setAttributeNS(null, 'height', 40);
		}
		rect3.setAttributeNS(null, 'fill', 'tomato');
		rect3.setAttributeNS(null, 'fill-opacity', '0.6');
		chartGroup.appendChild(rect3);
	}

	if (rate > 10000) {
		// round result to 1 decimal
		rate = Math.round(rate / 1000 * 10) / 10;
		unit = 'kbit/s';
	}

	if (rate > 10000) {
		// round result to 1 decimal
		rate = Math.round(rate / 1000 * 10) / 10;
		unit = 'mbit/s';
	}

	if (rxrate > 10000) {
		// round result to 1 decimal
		rxrate = Math.round(rxrate / 1000 * 10) / 10;
		rxunit = 'kbit/s';
	}

	if (rxrate > 10000) {
		// round result to 1 decimal
		rxrate = Math.round(rxrate / 1000 * 10) / 10;
		rxunit = 'mbit/s';
	}

	if (txrate > 10000) {
		// round result to 1 decimal
		txrate = Math.round(txrate / 1000 * 10) / 10;
		txunit = 'kbit/s';
	}

	if (txrate > 10000) {
		// round result to 1 decimal
		txrate = Math.round(txrate / 1000 * 10) / 10;
		txunit = 'mbit/s';
	}

	if (countera['maxrxrate'] > 0) {
		text1 = documentSVG.createElementNS(svgNS, "text");
		text1.setAttributeNS(null, 'x', 10);
		if (countera['maxtxrate'] > 0) {
			text1.setAttributeNS(null, 'y', 15);
		} else {
			text1.setAttributeNS(null, 'y', 25);
		}
		chartGroup.appendChild(text1);
		text1.appendChild(documentSVG.createTextNode(rxrate + ' ' +
							     rxunit + ' (RX)'));
	}

	if (countera['maxtxrate'] > 0) {
		text2 = documentSVG.createElementNS(svgNS, "text");
		text2.setAttributeNS(null, 'x', 10);
		if (countera['maxrxrate'] > 0) {
			text2.setAttributeNS(null, 'y', 35);
		} else {
			text2.setAttributeNS(null, 'y', 25);
		}
		chartGroup.appendChild(text2);
		text2.appendChild(documentSVG.createTextNode(txrate + ' ' +
							     txunit + ' (TX)'));
	}

	// Run interface counter query after updateInterval again
	setTimeout('do_interface_counter();', updateInterval);

}

function do_interface_counter() {
	device = get_GET_param('ethername');
	x_rrdtool_counter(device, do_interface_counter_cb);
}

function do_rrdtool_update_port_dbs_db() {
	do_rrdtool_update_port_dbs();
}

function do_rrdtool_update_port_dbs() {
	device = get_GET_param('ethername');
	x_rrdtool_update_port_dbs(device, do_rrdtool_update_port_dbs_db);
}
