// SVG namespace
var svgNS = "http://www.w3.org/2000/svg";
// the SVG document handler
var documentSVG = null;
// will store the root <g> element that groups all chart elements
var chartGroup = null;
// how often to request new data from server?
var updateInterval = 3000;
// coordinates (in pixels) used to translate the chart
var x = 50, y = 30;
// chart's dimension (in pixels)
var height = 300, width = 460;
// chart's axis origin
var xt1 = 0, yt1 = 0;
// chart's axis maximum values
var xt2 = 60, yt2 = 100;
// number of horizontal and vertical axis divisions
var xDivisions = 10, yDivisions = 10;
// default text width and height for initial display (recalculated
// afterwards)
var defaultTextWidth = 30, defaultTextHeight = 20;
// will retain references to the chart units for recalculating positions
var xIndexes = new Array(xDivisions + 1);
var yIndexes = new Array(yDivisions + 1);
// will store the text node that displays the selected chart node
var currentNodeInfo;
// shared svg elements
var chartGroup, dataGroup, dataPath;
// will store the text node for the clock
var currentWatch;
// our counter
var counter = 0;
// first run
var first = new Boolean(true);
// save last minute
var lastMin = -1;

// initializes chart
function init(evt)
{
	/**** Prepare the group that will contain all chart data  ****/
	// obtain SVG document handler
	documentSVG = evt.target.ownerDocument;
	// create the <g> element that groups all chart elements
	chartGroup = documentSVG.createElementNS(svgNS, "g");
	chartGroup.setAttribute("transform", "translate(" + x + " " + y + ")");

	/*** Prepare the group that will store the Y and Y axis and numbers ***/
	axisGroup = documentSVG.createElementNS(svgNS, "g");
	// create the X axis line as a <path> element
	axisPath = documentSVG.createElementNS(svgNS, "path");
	// the axis lines will be black, 2 pixels wide
	axisPath.setAttribute("stroke", "black");
	axisPath.setAttribute("stroke-width", "2");

	/**** Create the division lines for the X and Y axis  ****/
	// create the path definition text for the X axis division lines
	pathText = "M 0 " + height;
	// adds divisions to the X axis (differently for last division)
	for (var i = 0; i <= xDivisions; i++)
		pathText += "l 0 5 l 0 -5 " +
			((i == xDivisions) ? "" :
			("l " + width/xDivisions + " 0"));
	// create the path definition text for the Y axis division lines
	pathText += "M 0 " + height;
	// adds one division to the Y axis (differently for last division)
	for (var i = 0; i <= yDivisions; i++)
		pathText += "l -5 0 l 5 0 " +
			((i == yDivisions) ? "" :
			("l 0 -" + height / yDivisions));
	// add the path definition (the <d> attribute) to the path
	axisPath.setAttribute("d", pathText);
	// add the path to the axis group
	axisGroup.appendChild(axisPath);

	/**** Create the text nodes for the X and Y axis  ****/
	// adds text nodes for the X axis
	for (var i = 0; i <= xDivisions; i++)
	{
		// creates the <text> node for the division
		t = documentSVG.createElementNS(svgNS, "text");
		// stores the node for future reference
		xIndexes[i] = t;
		// creates the text for the <text> node
		t.appendChild(documentSVG.createTextNode(
				(xt1 + i * ((xt2 - xt1) / xDivisions)) + 's'));
		// sets the X and Y attributes for the <text> node
		t.setAttribute("x", i * width / xDivisions -
				defaultTextWidth / 2);
		t.setAttribute("y", height + 30 + defaultTextHeight);
		// when the graph first loads, we want the text nodes invisible
		t.setAttribute("stroke", "white");
		// TODO
		//t.setAttribute("writing-mode", "tb");
		// add the <text> node to the axis group
		axisGroup.appendChild(t);
	}
	// adds text nodes for the Y axis
	for (var i = 0; i <= yDivisions; i++)
	{
		// creates the <text> node for the division
		t = documentSVG.createElementNS(svgNS, "text");
		// stores the node for future reference
		yIndexes[i] = t;
		// creates the text for the <text> node
		t.appendChild(documentSVG.createTextNode(
				 (yt1 + i * ((yt2 - yt1) / yDivisions)) + '%'));

		// sets the X and Y attributes for the <text> node
		t.setAttribute("x", -30 -defaultTextWidth);
		t.setAttribute("y", height - i * height / yDivisions
				+ defaultTextHeight / 2);
		// when the graph first loads, we want the text nodes invisible
		t.setAttribute("stroke", "white");
		// add the <text> node to the axis group
		axisGroup.appendChild(t);
	}

	// add the axis group to the chart
	chartGroup.appendChild(axisGroup);

	/**** Prepare the <path> element that will draw chart's data ****/
	// Total
	dataPath = documentSVG.createElementNS(svgNS, "path");
	dataPath.setAttribute("stroke", "blue");
	dataPath.setAttribute("stroke-width", "1");
	dataPath.setAttribute("fill", "none");
	// add the data path to the chart group
	chartGroup.appendChild(dataPath);
	// RX
	dataPathRX = documentSVG.createElementNS(svgNS, "path");
	dataPathRX.setAttribute("stroke", "palegreen");
	dataPathRX.setAttribute("stroke-width", "1");
	dataPathRX.setAttribute("fill", "none");
	// add the data path to the chart group
	// TX
	chartGroup.appendChild(dataPathRX);
	dataPathTX = documentSVG.createElementNS(svgNS, "path");
	dataPathTX.setAttribute("stroke", "tomato");
	dataPathTX.setAttribute("stroke-width", "1");
	dataPathTX.setAttribute("fill", "none");
	// add the data path to the chart group
	chartGroup.appendChild(dataPathTX);

	// print clock at end of graph
	displayWatch(new Date());

	/**** Final initialization steps ****/
	// add the chart group to the SVG document
	documentSVG.documentElement.appendChild(chartGroup);
	// this is needed to correctly display text nodes in Firefox
	setTimeout("refreshXYIndexes()", 500);
	// update rrd databases
	do_rrdtool_update_port_dbs();
	// initiate repetitive server requests
	setTimeout("do_interface_counter()", updateInterval);
}

// this function redraws the text for the axis units and makes it visible
// (this is required to correctly position the text in Firefox)
function refreshXYIndexes()
{
	// redraw text nodes on the X axis
	for (var i = 0; i <= xDivisions; i++)
		if (typeof xIndexes[i].getBBox != "undefined")
			try
			{
				textWidth = xIndexes[i].getBBox().width;
				textHeight = xIndexes[i].getBBox().height;
				xIndexes[i].setAttribute("x",
					i*width/xDivisions - textWidth/2);
				xIndexes[i].setAttribute("y",
					height + 10 + textHeight);
				xIndexes[i].setAttribute("stroke", "black");
				// TODO
				//xIndexes[i].setAttribute("writing-mode", "tb");
			}
			catch(e) {}
	// redraw text nodes on the Y axis
	for (var i = 0; i <= yDivisions; i++)
		if (typeof yIndexes[i].getBBox != "undefined")
			try
			{
				twidth = yIndexes[i].getBBox().width;
				theight = yIndexes[i].getBBox().height;
				yIndexes[i].setAttribute("y",
					height-i*height/yDivisions+theight/2);
				yIndexes[i].setAttribute("x", -10 -twidth);
				yIndexes[i].setAttribute("stroke", "black");
			}
			catch(e) {}
}


// called when mouse hovers over chart node to display its coordinates
function createPointInfo(x, y, whereX, whereY)
{
	// make sure you don't display more coordinates at the same time
	if (currentNodeInfo) removePointInfo();
	// create text node
	currentNodeInfo = documentSVG.createElementNS(svgNS, "text");
	currentNodeInfo.appendChild(documentSVG.createTextNode("("+x+","+y+")"));
	// set coordinates
	currentNodeInfo.setAttribute("x", whereX.toFixed(1));
	currentNodeInfo.setAttribute("y", whereY - 10);
	// add the node to the group
	chartGroup.appendChild(currentNodeInfo);
}

// removes the text node that displays chart node coordinates
function removePointInfo()
{
	chartGroup.removeChild(currentNodeInfo);
	currentNodeInfo = null;
}

// draws a new point on the graph
function addPoint(X, Y)
{
	// start over (reload page) after the last value was generated
	if (X == xt2)
		window.location.reload(false);
	// calculate the coordinates of the new node
	coordX = (X - xt1) * (width / (xt2 - xt1));
	coordY = height - (Y - yt1) * (height / (yt2 - yt1));
	// draw the node on the chart as a blue filled circle
	var circle = documentSVG.createElementNS(svgNS, "circle");
	circle.setAttribute("cx", coordX); // X position
	circle.setAttribute("cy", coordY); // Y position
	circle.setAttribute("r", 3); // radius
	circle.setAttribute("fill", "blue"); // color
	circle.setAttribute("onmouseover",
			     "createPointInfo(" + X + "," +
			      Y + "," + coordX + "," + coordY + ")");
	circle.setAttribute("onmouseout", "removePointInfo()");
	chartGroup.appendChild(circle);
	// add a new line to the new node on the graph
	current = dataPath.getAttribute("d"); // current path definition
	// update path definition
	if (!current || current == "")
		dataPath.setAttribute("d", " M " + coordX + " " + coordY);
	else
		dataPath.setAttribute("d", current + " L " + coordX + " " +
					coordY);
}

function addPointRX(X, Y)
{
	// start over (reload page) after the last value was generated
	if (X == xt2)
		window.location.reload(false);
	// calculate the coordinates of the new node
	coordX = (X - xt1) * (width / (xt2 - xt1));
	coordY = height - (Y - yt1) * (height / (yt2 - yt1));
	// draw the node on the chart as a blue filled circle
	var circle = documentSVG.createElementNS(svgNS, "circle");
	circle.setAttribute("cx", coordX); // X position
	circle.setAttribute("cy", coordY); // Y position
	circle.setAttribute("r", 3); // radius
	circle.setAttribute("fill", "palegreen"); // color
	circle.setAttribute("onmouseover",
			     "createPointInfo(" + X + "," +
			      Y + "," + coordX + "," + coordY + ")");
	circle.setAttribute("onmouseout", "removePointInfo()");
	chartGroup.appendChild(circle);
	// add a new line to the new node on the graph
	current = dataPathRX.getAttribute("d"); // current path definition
	// update path definition
	if (!current || current == "")
		dataPathRX.setAttribute("d", " M " + coordX + " " + coordY);
	else
		dataPathRX.setAttribute("d", current + " L " + coordX + " " +
					coordY);
}

// draws a new point on the graph
function addPointTX(X, Y)
{
	// start over (reload page) after the last value was generated
	if (X == xt2)
		window.location.reload(false);
	// calculate the coordinates of the new node
	coordX = (X - xt1) * (width / (xt2 - xt1));
	coordY = height - (Y - yt1) * (height / (yt2 - yt1));
	// draw the node on the chart as a blue filled circle
	var circle = documentSVG.createElementNS(svgNS, "circle");
	circle.setAttribute("cx", coordX); // X position
	circle.setAttribute("cy", coordY); // Y position
	circle.setAttribute("r", 3); // radius
	circle.setAttribute("fill", "tomato"); // color
	circle.setAttribute("onmouseover",
			    "createPointInfo(" + X + "," +
					Y + "," + coordX + "," + coordY + ")");
	circle.setAttribute("onmouseout", "removePointInfo()");
	chartGroup.appendChild(circle);
	// add a new line to the new node on the graph
	current = dataPathTX.getAttribute("d"); // current path definition
	// update path definition
	if (!current || current == "")
		dataPathTX.setAttribute("d", " M " + coordX + " " + coordY);
	else
		dataPathTX.setAttribute("d", current + " L " + coordX + " " +
					coordY);
}

// called when mouse hovers over chart node to display its coordinates
function displayWatch(time)
{
	var hrs = time.getHours();
	var min = time.getMinutes();
	var sec = time.getSeconds();
	if (min < 10) {
		min = '0' + min;
	}
	if (sec < 10) {
		sec = '0' + sec;
	}
	var clock = hrs + ':' + min + ':' +sec;

	// make sure you don't display more clocks at the same time
	if (currentWatch) {
		chartGroup.removeChild(currentWatch);
		currentWatch = null;
	}

	// create text node
	currentWatch = documentSVG.createElementNS(svgNS, "text");
	currentWatch.appendChild(documentSVG.createTextNode('Last update: ' +
							    clock));
	// set coordinates
	currentWatch.setAttribute("x", 150);
	currentWatch.setAttribute("y", 350);
	// add the node to the group
	chartGroup.appendChild(currentWatch);
}

function do_interface_counter_cb(countera) {

	// countera['last'] = unixtimestemp from last update
	var time = new Date(countera['last']*1000);
	var hrs = time.getHours();
	var min = time.getMinutes();
	var sec = time.getSeconds();
	//if (lastMin == -1) {
	//	lastMin = min;
	//} else {
	//	if (min != lastMin) {
	//		lastMin = min;
	//		window.location.reload(false);
	//	}
	//}

	displayWatch(time);
	if (first == true) {
		first = false;
	} else {
		counter += updateInterval/1000;
	}

	sec = counter;

	addPoint(sec, countera['percent']);
	if (countera['maxrxrate'] > 0)
		addPointRX(sec, countera['rxpercent']);
	if (countera['maxtxrate'] > 0)
		addPointTX(sec, countera['txpercent']);

	// Run interface counter query every updateInterval
	setTimeout('do_interface_counter();', updateInterval);

}

function do_interface_counter() {

	//var today = new Date();
	//var hrs = today.getHours();
	//var min = today.getMinutes();
	//var sec = today.getSeconds();
	//var update = sec % 3;
	//if (update == 0)
	//	sec = 1;
	device = get_GET_param('ethername');
	x_rrdtool_counter(device, do_interface_counter_cb);
}

function do_rrdtool_update_port_dbs_db() {
	do_rrdtool_update_port_dbs();
}

function do_rrdtool_update_port_dbs() {
	device = get_GET_param('ethername');
	x_rrdtool_update_port_dbs(device, do_rrdtool_update_port_dbs_db);
}
