#!/usr/bin/php
<?php
/*
 * Copyright (C) 2009-2012 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

$dirname = dirname($_SERVER['SCRIPT_FILENAME']);
require_once($dirname . '/../htdocs/include/conf.inc.php');
require_once($dirname . '/../functions/functions.inc.php');

$cfg = array();

setCfgfile(getBasedir() . '/script/' . getCfgfile());

// Disable DEBUG messages
$DEBUG = 0;

// Initialization of dastralyse
initialization();

$includes = array();
$excludes = array();

$year  = date("Y", strtotime("yesterday"));
$month = date("m", strtotime("yesterday"));
$day   = date("d", strtotime("yesterday"));

$timestamp = $year . '-' . $month . '-' . $day . ' 00:00:00';

$table = 'acct_v5_' . $year . $month . $day;

$table_yr = 'acct_' . $year;

$query_file = getBaseDir() . '/conf/query.php';

// Get configured network
foreach (getNetworks() as $network) {
	$net = $network['net'];
	if (isset($network['exclude'])) {
		array_push($excludes, $net);
	} else {
		array_push($includes, $net);
	}
}

// Check if the yearly table exists
$query  = "select * from pg_tables where schemaname='public' ";
$query .= " and tablename='". $table_yr . "'";

if (isset($query)) {
	debug_msg(2, '$query = ' . $query);
	$results = db_query($query);
}

if (isset($results) && is_array($results)) {
	// Query was successfull, now check if we have a result
	if ($results) {
		debug_msg(1, "found table '" . $table_yr . "'");
	} else {
		// Create the new table
		$query  = "create table " . $table_yr . "(";
		$query .= "timestamp timestamp without time zone NOT NULL";
		$query .= " DEFAULT '0001-01-01 00:00:00',";
		$query .= "ip inet NOT NULL DEFAULT '0.0.0.0',";
		$query .= "inbytes BIGINT NOT NULL,";
		$query .= "outbytes BIGINT NOT NULL,";
		$query .= "PRIMARY KEY(timestamp, ip)";
		$query .= ")";
		debug_msg(2, '$query = ' . $query);
		$results = db_query($query);
		if (isset($results) && is_array($results)) {
			debug_msg(1, "created table '" . $table_yr . "'");
		} else {
			error_msg("cannot create table '" . $table_yr . "'");
		}
	}
} else {
	error_msg("cannot check if table '" . $table_yr . "' exists");
}

// Build query start
$query = 'insert into ' . $table_yr . ' select';
// The timestamp is need for the new table, because this and the ip address
// is the primary key to only have one entry per day and ip address
$query .= " date_trunc('day', TIMESTAMP '" . $timestamp . "') as timestamp,";
$query .= ' x.ip,sum(x.inbytes) as inbytes, sum(x.outbytes) as outbytes from ';
$query .= '(select ip_src as ip, 0 as inbytes, bytes as outbytes ';
$query .= 'from ' . $table . ' ';

$count_exc = count($excludes);
if ($count_exc >= 1) {
	$query .= 'where ';
	foreach ($excludes as $key => $exclude) {
		$query .= "not ip_src<<'" . $exclude . "' and ";
		$query .= "not ip_dst<<'" . $exclude . "' ";
		if ($key < ($count_exc-1)) {
			$query .= 'and ';
		}
	}
}

$query .= 'union all select ip_dst as ip, bytes as inbytes, 0 as outbytes ';
$query .= 'from ' . $table . ' ';

if ($count_exc >= 1) {
	$query .= 'where ';
	foreach ($excludes as $key => $exclude) {
		$query .= "not ip_src<<='" . $exclude . "' and ";
		$query .= "not ip_dst<<='" . $exclude . "' ";
		if ($key < ($count_exc-1)) {
			$query .= 'and ';
		}
	}
}

$query .= ') as x ';

$count_inc = count($includes);
if ($count_inc >= 1) {
	$query .= 'where ';
	foreach ($includes as $key => $include) {
		$query .= "ip<<'" . $include . "' ";
		if ($key < ($count_inc-1)) {
			$query .= 'or ';
		}
	}
}

$query .= 'group by ip';

// Build query end

// Query can be overwritten by $query_file
if (is_readable($query_file)) {
	include($query_file);
} else {
	if (is_file($topfile)) {
		error_msg($query_file . ': cannot read file');
	}
}

if (isset($query)) {
	debug_msg(2, '$query = ' . $query);
	$results = db_query($query);
}

// Print debug messages
create_debug_console_msg($debug_msga);

// Print error messages
create_error_console_msg($error_msga);

?>
