#!/usr/bin/php
<?php
/*
 * Copyright (C) 2009-2016 Oliver Seufer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

$dirname = dirname($_SERVER['SCRIPT_FILENAME']);
require_once($dirname . '/../htdocs/include/conf.inc.php');
require_once($dirname . '/../functions/functions.inc.php');

$cfg = array();

setCfgfile(getBasedir() . '/script/' . getCfgfile());

// Disable DEBUG messages
$DEBUG = 0;

// Initialization of dastralyse
initialization();

$limit = getTopLimit();
$topfile = getBaseDir() . '/stats/toptraffic';
$topweekfile = getBaseDir() . '/stats/toptraffic-week';
$topmonthfile = getBaseDir() . '/stats/toptraffic-month';

$year  = date("Y", strtotime("yesterday"));
$month = date("m", strtotime("yesterday"));
$day   = date("d", strtotime("yesterday"));

$date = $day . '.' . $month . '.' . $year;

$timestamp = $year . '-' . $month . '-' . $day . ' 00:00:00';

$table_yr = 'acct_' . $year;

// Check if the yearly table exists
$query  = "select * from pg_tables where schemaname='public' ";
$query .= " and tablename='". $table_yr . "'";

if (isset($query)) {
	debug_msg(2, '$query = ' . $query);
	$results = db_query($query);
}

if (isset($results) && is_array($results)) {
	// Query was successfull, now check if we have a result
	if ($results) {
		debug_msg(1, "found table '" . $table_yr . "'");
	} else {
		error_msg("table '" . $table_yr . "' does not exist");
	}
} else {
	error_msg("cannot check if table '" . $table_yr . "' exists");
}

// Query the entries from the yearly database for the daily top stats
$query  = 'select ip,sum(inbytes+outbytes) as bytes from ' . $table_yr;
$query .= " where timestamp='" . $timestamp . "'";
$query .= " group by ip order by bytes desc";

if (isset($limit)) {
	$query .= ' limit ' . $limit;
}

if (isset($query)) {
	debug_msg(2, '$query = ' . $query);
	$results = db_query($query);
}

if (isset($results) && is_array($results)) {
	$output  = 'TOP ' . $limit . ' vom ' . $date . "\n";
	$output .= '(IP-Adressen nach Traffic sortiert)' . "\n\n";
	$output .= "       ip        |     traffic     | DNS\n";
	$output .= "-----------------+-----------------+---------------------\n";
	foreach($results as $value) {
		$ip = $value['ip'];
		$bytes = number_format($value['bytes'], 0, ',', '.');
		$dns = gethostbyaddr($ip);
		$output .= sprintf(" %-15s | %15s | %s \n", $ip, $bytes,
				    $dns);
	}
	//print($output);
	file_put_contents($topfile, $output);
} else {
	error_msg('No results');
}

// Weekly

$year  = date("Y", strtotime("-1 week -1 day"));
$month = date("m", strtotime("-1 week -1 day"));
$day   = date("d", strtotime("-1 week -1 day"));

$date_start = $day . '.' . $month . '.' . $year;
$timestamp_start = $year . '-' . $month . '-' . $day . ' 00:00:00';

$year  = date("Y", strtotime("yesterday"));
$month = date("m", strtotime("yesterday"));
$day   = date("d", strtotime("yesterday"));

$date_stop = $day . '.' . $month . '.' . $year;
$timestamp_stop = $year . '-' . $month . '-' . $day . ' 00:00:00';

// Query the entries from the yearly database for the weekly top stats
$query  = 'select ip,sum(inbytes+outbytes) as bytes from ' . $table_yr;
$query .= " where timestamp>='" . $timestamp_start . "'";
$query .= " and timestamp<='" . $timestamp_stop  ."'";
$query .= " group by ip order by bytes desc";

if (isset($limit)) {
	$query .= ' limit ' . $limit;
}

if (isset($query)) {
	debug_msg(2, '$query = ' . $query);
	$results = db_query($query);
}

if (isset($results) && is_array($results)) {
	$output  = 'TOP ' . $limit . ' vom ' . $date_start . ' bis ' . $date_stop . "\n";
	$output .= '(IP-Adressen nach Traffic sortiert)' . "\n\n";
	$output .= "       ip        |     traffic     | DNS\n";
	$output .= "-----------------+-----------------+---------------------\n";
	foreach($results as $value) {
		$ip = $value['ip'];
		$bytes = number_format($value['bytes'], 0, ',', '.');
		$dns = gethostbyaddr($ip);
		$output .= sprintf(" %-15s | %15s | %s \n", $ip, $bytes,
				    $dns);
	}
	//print($output);
	file_put_contents($topweekfile, $output);
} else {
	error_msg('No results');
}

// Monthly

$year  = date("Y", strtotime("-1 month -1 day"));
$month = date("m", strtotime("-1 month -1 day"));
$day   = date("d", strtotime("-1 month -1 day"));

$date_start = $day . '.' . $month . '.' . $year;
$timestamp_start = $year . '-' . $month . '-' . $day . ' 00:00:00';

$year  = date("Y", strtotime("yesterday"));
$month = date("m", strtotime("yesterday"));
$day   = date("d", strtotime("yesterday"));

$date_stop = $day . '.' . $month . '.' . $year;
$timestamp_stop = $year . '-' . $month . '-' . $day . ' 00:00:00';

// Query the entries from the yearly database for the monthly top stats
$query  = 'select ip,sum(inbytes+outbytes) as bytes from ' . $table_yr;
$query .= " where timestamp>='" . $timestamp_start . "'";
$query .= " and timestamp<='" . $timestamp_stop  ."'";
$query .= " group by ip order by bytes desc";

if (isset($limit)) {
	$query .= ' limit ' . $limit;
}

if (isset($query)) {
	debug_msg(2, '$query = ' . $query);
	$results = db_query($query);
}

if (isset($results) && is_array($results)) {
	$output  = 'TOP ' . $limit . ' vom ' . $date_start . ' bis ' . $date_stop . "\n";
	$output .= '(IP-Adressen nach Traffic sortiert)' . "\n\n";
	$output .= "       ip        |     traffic     | DNS\n";
	$output .= "-----------------+-----------------+---------------------\n";
	foreach($results as $value) {
		$ip = $value['ip'];
		$bytes = number_format($value['bytes'], 0, ',', '.');
		$dns = gethostbyaddr($ip);
		$output .= sprintf(" %-15s | %15s | %s \n", $ip, $bytes,
				    $dns);
	}
	//print($output);
	file_put_contents($topmonthfile, $output);
} else {
	error_msg('No results');
}

// Print debug messages
create_debug_console_msg($debug_msga);

// Print error messages
create_error_console_msg($error_msga);

?>
